@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 1997-2009 Torsten Rohlfing
+//  Copyright 1997-2012 Torsten Rohlfing
 //
 //  Copyright 2004-2012 SRI International
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3698 $
+//  $Revision: 4237 $
 //
-//  $LastChangedDate: 2012-01-06 10:19:25 -0800 (Fri, 06 Jan 2012) $
+//  $LastChangedDate: 2012-04-20 14:55:20 -0700 (Fri, 20 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -46,6 +46,8 @@
 #include <IO/cmtkVolumeFromFile.h>
 #include <IO/cmtkFileFormat.h>
 
+#include <mxml.h>
+
 #ifndef CMTK_USE_DCMTK
 #error Build system is broken: this application should not be build if CMTK_USE_DCMTK is not set.
 #endif
@@ -58,11 +60,14 @@
 #  include <unistd.h>
 #endif
 
+#include <ctype.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <vector>
 #include <map>
+#include <algorithm>
 #include <string>
 #include <sstream>
 
@@ -78,11 +83,15 @@
 #define DCM_RawDataType_ImageType DcmTagKey(0x0043,0x102f)
 #endif
 
+#ifndef DCM_ManufacturerModelName
+#define DCM_ManufacturerModelName DcmTagKey(0x0008,0x1090)
+#endif
+
 #ifndef DCM_PatientsName
 #define DCM_PatientsName DCM_PatientName
 #endif
 
-const char* OutPathPattern = "%03d.nii";
+const char* OutPathPattern = "image%n.nii";
 std::vector<std::string> SearchRootDirVector;
 
 std::ofstream cnull( "/dev/null" );
@@ -91,10 +100,21 @@ const char progress_chars[] = "-\\|/";
 int progress = 0;
 
 bool Recursive = false;
-int SortFiles = 1;
 
-bool WithExtensionsGE = false;
-const char *const GERawDataTypeString[4] = { "magn", "phas", "real", "imag" };
+/// Enum type to select sort order key.
+typedef enum
+{
+  /// No sorting - take files in order stored in file system.
+  SORT_NONE = 0,
+  /// Sort by file name.
+  SORT_FILENAME = 1,
+  /// Sort by instance number.
+  SORT_INSTANCE_NUMBER = 2
+} SortKeyEnum;
+
+SortKeyEnum SortFiles = SORT_INSTANCE_NUMBER;
+
+bool WriteXML = false;
 
 bool DisableOrientationCheck = false;
 double Tolerance = 1e-5;
@@ -117,7 +137,8 @@ typedef enum
 /// Selector for embedded image information.
 EmbedInfoEnum EmbedInfo = EMBED_STUDYID_STUDYDATE;
 
-class ImageFileDCM 
+/// Class handling a single image file and its meta data.
+class ImageFile 
 {
 public:
   /// File name.
@@ -126,6 +147,15 @@ public:
   /// File system path (i.e., directory).
   char* fpath;
 
+  /// Modality.
+  std::string Modality;
+
+  /// Device manufacturer.
+  std::string Manufacturer;
+
+  /// Device model.
+  std::string ManufacturerModel;
+
   /// Flag for multislice images
   bool IsMultislice;
 
@@ -165,47 +195,68 @@ public:
   /// DICOM image number (index in volume).
   Sint32 InstanceNumber;
 
-  /// GE private DICOM tag: raw data type (real, imaginary, phase, magnitude).
-  Sint16 GERawDataType;
+  /// Raw data type (real, imaginary, phase, magnitude) currently supported on GE images only.
+  std::string RawDataType;
+
+  /// Flag for diffusion-weighted images.
+  bool IsDWI;
+
+  /// B value for DWI.
+  Sint16 BValue;
+
+  /// B vector for DWI.
+  cmtk::FixedVector<3,double> BVector;
 
   /// Constructor.
-  ImageFileDCM( const char* filename );
+  ImageFile( const char* filename );
 
   /// Destructor.
-  ~ImageFileDCM();
+  ~ImageFile();
 
   /// Determine whether two images match, i.e., belong to the same volume.
-  bool Match( const ImageFileDCM& other ) const;
+  bool Match( const ImageFile& other ) const;
 
   /// Compare order based on file name (for lexicographic sorting).
-  static bool lessFileName( const ImageFileDCM* lhs, const ImageFileDCM* rhs )
+  static bool lessFileName( const ImageFile* lhs, const ImageFile* rhs )
   {
     return strcmp( lhs->fname, rhs->fname ) < 0;
   }
 
   /// Compare order based on image instace (for sorting in acquisition order).
-  static bool lessInstanceNumber( const ImageFileDCM* lhs, const ImageFileDCM* rhs )
+  static bool lessInstanceNumber( const ImageFile* lhs, const ImageFile* rhs )
   {
     return lhs->InstanceNumber < rhs->InstanceNumber;
   }
 
   void Print() const;
+
+private:
+  /// Handle Siemens private tags.
+  void DoVendorTagsSiemens( const DiDocument& document );
+
+  /// Handle GE private tags.
+  void DoVendorTagsGE( const DiDocument& document );
 };
 
 void
-ImageFileDCM::Print() const
+ImageFile::Print() const
 {
-  cmtk::DebugOutput( 1 ) << "  File Name = [" << this->fpath << "/" << this->fname << "]\n";
-  cmtk::DebugOutput( 1 ) << "  SeriesID =    [" << this->SeriesUID << "]\n";
-  cmtk::DebugOutput( 1 ) << "  StudyID =       [" << this->StudyUID << "]\n";
-  cmtk::DebugOutput( 1 ) << "  EchoTime =        [" << this->EchoTime << "]\n";
-  cmtk::DebugOutput( 1 ) << "  RepetitionTime =    [" << this->RepetitionTime << "]\n";
-  cmtk::DebugOutput( 1 ) << "  AcquisitionNumber =   [" << this->AcquisitionNumber << "]\n";
-  cmtk::DebugOutput( 1 ) << "  ImagePositionPatient =  [" << this->ImagePositionPatient << "]\n";
+  cmtk::DebugOutput( 1 ) << "  File Name =            [" << this->fpath << "/" << this->fname << "]\n";
+  cmtk::DebugOutput( 1 ) << "  SeriesID =             [" << this->SeriesUID << "]\n";
+  cmtk::DebugOutput( 1 ) << "  StudyID =              [" << this->StudyUID << "]\n";
+  cmtk::DebugOutput( 1 ) << "  ImagePositionPatient = [" << this->ImagePositionPatient << "]\n";
+  cmtk::DebugOutput( 1 ) << "  AcquisitionNumber =    [" << this->AcquisitionNumber << "]\n";
+  cmtk::DebugOutput( 1 ) << "  Modality =             [" << this->Modality << "]\n";
+
+  if ( this->Modality == "MR" )
+    {
+    cmtk::DebugOutput( 1 ) << "  EchoTime =          [" << this->EchoTime << "]\n";
+    cmtk::DebugOutput( 1 ) << "  RepetitionTime =      [" << this->RepetitionTime << "]\n";
+    }
 }
   
 bool
-ImageFileDCM::Match( const ImageFileDCM& other ) const
+ImageFile::Match( const ImageFile& other ) const
 {
   // do not stack multislice images
   if ( this->IsMultislice || other.IsMultislice )
@@ -229,12 +280,21 @@ ImageFileDCM::Match( const ImageFileDCM& other ) const
     ( StudyUID == other.StudyUID ) && 
     ( EchoTime == other.EchoTime ) &&
     ( RepetitionTime == other.RepetitionTime ) && 
+    ( BValue == other.BValue ) &&
+    ( BVector == other.BVector ) &&
     (( AcquisitionNumber == other.AcquisitionNumber ) || IgnoreAcquisitionNumber) && 
-    ( this->GERawDataType == other.GERawDataType );
+    ( this->RawDataType == other.RawDataType );
 }
 
-ImageFileDCM::ImageFileDCM( const char* filename )
-  : IsMultislice( false )
+ImageFile::ImageFile( const char* filename )
+  : Modality( "unknown" ),
+    Manufacturer( "unknown" ),
+    ManufacturerModel( "unknown" ),
+    IsMultislice( false ),
+    RawDataType( "unknown" ),
+    IsDWI( false ),
+    BValue( 0 ),
+    BVector( cmtk::FixedVector<3,double>::Init( 0.0 ) )
 {
   if ( cmtk::FileFormat::Identify( filename, false /*decompress*/ ) != cmtk::FILEFORMAT_DICOM ) // need to disable "decompress" in Identify() because DCMTK cannot currently read using on-the-fly decompression.
     throw(0);
@@ -285,6 +345,10 @@ ImageFileDCM::ImageFileDCM( const char* filename )
     throw(2);
     }
 
+  const char* tmpStr = NULL;
+  if ( document->getValue( DCM_Modality, tmpStr ) )
+    this->Modality = tmpStr;
+
   // check for multi-slice DICOMs
   Uint16 nFrames = 0;
   if ( document->getValue( DCM_NumberOfFrames, nFrames ) ) 
@@ -292,17 +356,6 @@ ImageFileDCM::ImageFileDCM( const char* filename )
     this->IsMultislice = (nFrames > 1 );
     }
 
-  // check if this is a Siemens mosaic image with multiple slices
-  const char* tmpStr = NULL;
-  if ( document->getValue( DCM_Manufacturer, tmpStr ) )
-    {
-    if ( !strncmp( tmpStr, "SIEMENS", 7 ) )
-      {
-      const DcmTagKey nSlicesTag(0x0019,0x100a);
-      this->IsMultislice = document->getValue( nSlicesTag, nFrames );
-      }      
-    }
-  
   if ( document->getValue( DCM_PatientsName, tmpStr ) )
     PatientName = tmpStr;
 
@@ -321,12 +374,6 @@ ImageFileDCM::ImageFileDCM( const char* filename )
   if ( document->getValue( DCM_StudyDate, tmpStr ) )
     StudyDate = tmpStr;
   
-  if ( document->getValue( DCM_EchoTime, tmpStr ) )
-    EchoTime = tmpStr;
-
-  if ( document->getValue( DCM_RepetitionTime, tmpStr ) )
-    RepetitionTime = tmpStr;
-
   if ( document->getValue( DCM_ImagePositionPatient, tmpStr ) )
     ImagePositionPatient = tmpStr;
 
@@ -339,44 +386,160 @@ ImageFileDCM::ImageFileDCM( const char* filename )
   if ( ! document->getValue( DCM_AcquisitionNumber, AcquisitionNumber ) )
     AcquisitionNumber = 0;
 
-  if ( WithExtensionsGE )
+  // check for MR modality and treat accordingly
+  if ( this->Modality == "MR" )
     {
-    if ( ! document->getValue( DCM_RawDataType_ImageType, this->GERawDataType ) )
-      this->GERawDataType = 3; // assume this is a magnitude image
-    this->GERawDataType = std::min( 3, std::max( 0, (int)GERawDataType ) );
+    if ( document->getValue( DCM_EchoTime, tmpStr ) )
+      EchoTime = tmpStr;
+    
+    if ( document->getValue( DCM_RepetitionTime, tmpStr ) )
+      RepetitionTime = tmpStr;
     }
   else
     {
-    // GE extensions disabled
-    this->GERawDataType = 0;
+    this->EchoTime = this->RepetitionTime = "";
+    }
+  
+  if ( document->getValue( DCM_ManufacturerModelName, tmpStr ) != 0 )
+    this->ManufacturerModel = tmpStr;
+
+  // check for which vendor and deal with specifics elsewhere
+  if ( document->getValue( DCM_Manufacturer, tmpStr ) != 0 )
+    {
+    this->Manufacturer = tmpStr;
+    if ( !strncmp( tmpStr, "SIEMENS", 7 ) )
+      {
+      this->DoVendorTagsSiemens( *document );
+      }      
+    
+    if ( !strncmp( tmpStr, "GE", 2 ) )
+      {
+      this->DoVendorTagsGE( *document );
+      }
     }
 }
 
-ImageFileDCM::~ImageFileDCM()
+void
+ImageFile::DoVendorTagsSiemens( const DiDocument& document )
+{
+  Uint16 nFrames = 0;
+  const char* tmpStr = NULL;
+
+  this->IsMultislice = document.getValue( DcmTagKey (0x0019,0x100a), nFrames ); // Number of Slices tag
+  
+  if ( this->Modality == "MR" )
+    {
+    if ( (this->IsDWI = (document.getValue( DcmTagKey(0x0019,0x100d), tmpStr )!=0)) ) // "Directionality" tag
+      {
+      if ( document.getValue( DcmTagKey(0x0019,0x100c), tmpStr ) != 0 ) // bValue tag
+	{
+	this->BValue = atoi( tmpStr );
+	this->IsDWI |= (this->BValue > 0);
+	}
+      
+      if ( this->BValue > 0 )
+	{
+	for ( int idx = 0; idx < 3; ++idx )
+	  {
+	  this->IsDWI |= (document.getValue( DcmTagKey(0x0019,0x100e), this->BVector[idx], idx ) != 0);
+	  }
+	}
+      }
+    }
+}
+
+void
+ImageFile::DoVendorTagsGE( const DiDocument& document )
+{
+  const char* tmpStr = NULL;
+  int tmpInt = 0;
+
+  if ( this->Modality == "MR" )
+    {
+    // raw data type
+    Sint16 rawTypeIdx = 3;
+    if ( ! document.getValue( DCM_RawDataType_ImageType, rawTypeIdx ) )
+      rawTypeIdx = 3; // assume this is a magnitude image
+    rawTypeIdx = std::min( 3, std::max( 0, (int)rawTypeIdx ) );
+    
+    const char *const RawDataTypeString[4] = { "magn", "phas", "real", "imag" };
+    this->RawDataType = RawDataTypeString[rawTypeIdx];
+    
+    // dwi information
+    this->IsDWI = false;
+    if ( document.getValue( DcmTagKey(0x0019,0x10e0), tmpStr ) > 0 ) // Number of Diffusion Directions
+      {
+      const int nDirections = atoi( tmpStr );
+      if ( nDirections > 0 )
+	{
+	this->IsDWI = true;
+	
+	if ( document.getValue( DcmTagKey(0x0043,0x1039), tmpStr ) > 0 ) // bValue tag
+	  {
+	  if ( 1 == sscanf( tmpStr, "%d\\%*c", &tmpInt ) )
+	    {
+	    this->BValue = static_cast<Sint16>( tmpInt );
+
+	    for ( int i = 0; i < 3; ++i )
+	      {
+	      if ( document.getValue( DcmTagKey(0x0019,0x10bb+i), tmpStr ) > 0 ) // bVector tags
+		{
+		this->BVector[i] = atof( tmpStr );
+		}
+	      else
+		{
+		this->BVector[i] = 0;
+		}
+	      }
+	    this->BVector[2] *= -1; // for some reason z component apparently requires negation of sign on GE
+	    }
+	  }
+	}
+      }
+    }
+}
+
+ImageFile::~ImageFile()
 {
   free( fname );
   free( fpath );
 }
 
-class VolumeDCM : public std::vector<ImageFileDCM*> 
+/// Class handling a stack of image files.
+class ImageStack : public std::vector<ImageFile*> 
 {
 public:
-  void AddImageFileDCM( ImageFileDCM *const image );
+  /// This class.
+  typedef ImageStack Self;
   
-  bool Match ( const ImageFileDCM *newImage ) const;
+  /// Add new DICOM image file to this stack.
+  void AddImageFile( ImageFile *const image );
 
-  void WriteToArchive ( const std::string& name ) const;
+  /// Match new image file against this volume stack.
+  bool Match ( const ImageFile *newImage ) const;
 
+  /// Write XML sidecar file.
+  void WriteXML ( const std::string& name /*!< Sidecar XML file name. */, const cmtk::UniformVolume& volume /*!< Previously written image volume - provides information about coordinate system etc. */ ) const;
+
+  /// Write to image file.
+  cmtk::UniformVolume::SmartConstPtr WriteImage ( const std::string& name ) const;
+
+  /// Print stack information.
   void print() const;
+
+private:
+  /// Generate custom whitespaces for XML output.
+  static const char *WhitespaceWriteMiniXML( mxml_node_t*, int where);
+
 };
 
 bool
-VolumeDCM::Match ( const ImageFileDCM *newImage ) const
+ImageStack::Match ( const ImageFile *newImage ) const
 {
   if ( empty() ) 
     return 1;
 
-  const ImageFileDCM *check = front();
+  const ImageFile *check = front();
   if ( check )
     {
     if ( !check->Match( *newImage ) )
@@ -396,7 +559,7 @@ VolumeDCM::Match ( const ImageFileDCM *newImage ) const
 }
 
 void
-VolumeDCM::AddImageFileDCM ( ImageFileDCM *const newImage )
+ImageStack::AddImageFile ( ImageFile *const newImage )
 {
   iterator it = begin();
   for ( ; it != end(); ++it )
@@ -404,10 +567,165 @@ VolumeDCM::AddImageFileDCM ( ImageFileDCM *const newImage )
   insert( it, newImage );
 }
 
-void
-VolumeDCM::WriteToArchive( const std::string& fname ) const
+const char *
+ImageStack::WhitespaceWriteMiniXML( mxml_node_t* node, int where)
 {
-  const ImageFileDCM *first = this->front();
+  const char* name = node->value.element.name;
+  
+  typedef struct _wsLookupType
+  {
+    /// XML element name.
+    const char* name;
+    /// Table of whitespace sequences.
+    const char* ws[4];
+  } wsLookupType;
+
+  static const wsLookupType wsLookup[] = 
+  {
+    { "manufacturer",    { "\t", NULL, NULL, "\n" } },
+    { "model",           { "\t", NULL, NULL, "\n" } },
+    { "tr",              { "\t", NULL, NULL, "\n" } },
+    { "te",              { "\t", NULL, NULL, "\n" } },
+    { "dwi",             { "\t", "\n", "\t", "\n" } },
+    { "bValue",          { "\t\t", NULL, NULL, "\n" } },
+    { "bVector",         { "\t\t", NULL, NULL, "\n" } },
+    { "bVectorImage",    { "\t\t", NULL, NULL, "\n" } },
+    { "bVectorStandard", { "\t\t", NULL, NULL, "\n" } },
+    { "dcmPath",         { NULL, NULL, NULL, "\n" } },
+    { "dcmFile",         { "\t", NULL, NULL, "\n" } },
+    { NULL, {NULL, NULL, NULL, NULL} }
+  };
+
+  if ( (where >= 0) && (where < 4) )
+    {
+    for ( size_t idx = 0; wsLookup[idx].name; ++idx )
+      {
+      if ( ! strcmp( name, wsLookup[idx].name ) )
+	return wsLookup[idx].ws[where];
+      }
+    }
+
+  switch ( where )
+    {
+    case MXML_WS_BEFORE_OPEN:
+      return NULL;
+    case MXML_WS_AFTER_OPEN:
+      return "\n";
+    case MXML_WS_BEFORE_CLOSE:
+      return NULL;
+    case MXML_WS_AFTER_CLOSE:
+      return "\n";
+    }
+
+  return NULL;
+}
+
+// wrap tolower() - on Mac, system function is not compatible with std::transform()
+static int cmtkWrapToLower( const int c )
+{
+  return tolower( c );
+}
+
+void
+ImageStack::WriteXML( const std::string& fname, const cmtk::UniformVolume& volume ) const
+{
+  mxmlSetWrapMargin( 120 ); // make enough room for indented bVectorStandard
+  mxml_node_t *x_root = mxmlNewElement( NULL, "?xml version=\"1.0\" encoding=\"utf-8\"?" );
+
+  mxml_node_t *x_device = mxmlNewElement( x_root, "device" );
+
+  mxml_node_t *x_manufacturer = mxmlNewElement( x_device, "manufacturer" );
+  mxmlNewText( x_manufacturer, 0, this->front()->Manufacturer.c_str() );
+    
+  mxml_node_t *x_model = mxmlNewElement( x_device, "model" );
+  mxmlNewText( x_model, 0, this->front()->ManufacturerModel.c_str() );
+
+  std::string modality = this->front()->Modality;
+  std::transform( modality.begin(), modality.end(), modality.begin(), cmtkWrapToLower );
+  
+  mxml_node_t *x_modality = mxmlNewElement( x_root, modality.c_str() );
+  if ( modality == "mr" )
+    {
+    mxml_node_t *x_tr = mxmlNewElement( x_modality, "tr");
+    mxmlNewReal( x_tr, atof( this->front()->RepetitionTime.c_str() ) );
+    
+    mxml_node_t *x_te = mxmlNewElement( x_modality, "te");
+    mxmlNewReal( x_te, atof( this->front()->EchoTime.c_str() ) );
+    
+    if ( this->front()->IsDWI )
+      {
+      mxml_node_t *x_dwi = mxmlNewElement( x_modality, "dwi" );
+      
+      mxml_node_t *x_bval = mxmlNewElement( x_dwi, "bValue");
+      mxmlNewInteger( x_bval, this->front()->BValue );
+      
+      mxml_node_t *x_bvec = mxmlNewElement( x_dwi, "bVector");
+      mxmlElementSetAttr( x_bvec, "coordinateSpace", "LPS" );
+      for ( size_t idx = 0; idx < 3; ++idx )
+	{
+	mxmlNewReal( x_bvec, this->front()->BVector[idx] );
+	}
+
+      // Determine bVector in image LPS coordinate space:
+      // First, create copy of image grid
+      cmtk::UniformVolume::SmartPtr gridLPS = volume.CloneGrid();
+      // Make sure still in LPS DICOM coordinate space
+      gridLPS->ChangeCoordinateSpace( "LPS" );
+      // Apply inverse of remaining image-to-space matrix to original bVector
+      const cmtk::UniformVolume::CoordinateVectorType bVectorImage = this->front()->BVector * cmtk::Matrix3x3<cmtk::Types::Coordinate>( gridLPS->GetImageToPhysicalMatrix().GetInverse() );
+      
+      mxml_node_t *x_bvec_image = mxmlNewElement( x_dwi, "bVectorImage");
+      mxmlElementSetAttr( x_bvec_image, "imageOrientation", gridLPS->GetMetaInfo( cmtk::META_IMAGE_ORIENTATION ).c_str() );
+      for ( size_t idx = 0; idx < 3; ++idx )
+	{
+	mxmlNewReal( x_bvec_image, bVectorImage[idx] );
+	}
+
+      // Determine bVector in image RAS standard coordinate space:
+      // First, create copy of image grid
+      cmtk::UniformVolume::SmartPtr gridRAS = gridLPS->GetReoriented();
+      // Apply inverse of remaining image-to-space matrix to original bVector
+      const cmtk::UniformVolume::CoordinateVectorType bVectorStandard = this->front()->BVector * cmtk::Matrix3x3<cmtk::Types::Coordinate>( gridRAS->GetImageToPhysicalMatrix().GetInverse() );
+      
+      mxml_node_t *x_bvec_std = mxmlNewElement( x_dwi, "bVectorStandard");
+      mxmlElementSetAttr( x_bvec_std, "imageOrientation", gridRAS->GetMetaInfo( cmtk::META_IMAGE_ORIENTATION ).c_str() );
+      for ( size_t idx = 0; idx < 3; ++idx )
+	{
+	mxmlNewReal( x_bvec_std, bVectorStandard[idx] );
+	}
+      }
+    }
+    
+  mxml_node_t *x_dcmpath = mxmlNewElement( x_root, "dcmPath" );
+  mxmlNewText( x_dcmpath, 0, this->front()->fpath );
+
+  mxml_node_t *x_stack = mxmlNewElement( x_root, "stack" );
+
+  for ( const_iterator it = this->begin(); it != this->end(); ++it ) 
+    {
+    mxml_node_t *x_dcmfile = mxmlNewElement( x_stack, "dcmFile" );
+    mxmlNewText( x_dcmfile, 0, (*it)->fname );
+    }
+
+  FILE *file = fopen( fname.c_str(), "w" );
+  if ( file )
+    {
+    mxmlSaveFile( x_root, file, Self::WhitespaceWriteMiniXML );
+    fputs( "\n", file ); // end last line
+    fclose( file );
+    }
+  else
+    {
+    cmtk::StdErr << "ERROR: could not open file " << fname << " for writing\n";
+    }
+  
+  mxmlDelete( x_root );
+}
+
+cmtk::UniformVolume::SmartConstPtr
+ImageStack::WriteImage( const std::string& fname ) const
+{
+  const ImageFile *first = this->front();
     
   cmtk::UniformVolume::SmartPtr volume;
   if ( !first->IsMultislice )
@@ -466,18 +784,14 @@ VolumeDCM::WriteToArchive( const std::string& fname ) const
     }
   
   cmtk::DebugOutput( 1 ) << "DICOM Information: \n"
-			 << "  Description: " << first->SeriesDescription << "\n"
-			 << "  Series:      " << first->SeriesUID << "\n"
-			 << "  Study:       " << first->StudyUID << "\n"
-			 << "  Acquisition: " << first->AcquisitionNumber << "\n"
-			 << "  TR / TE:     " << first->RepetitionTime << "ms /" << first->EchoTime << "ms\n"
-			 << "  Position:    " << first->ImagePositionPatient << "\n"
-			 << "  Orientation: " << first->ImageOrientationPatient << "\n";
-  
-  if ( WithExtensionsGE )
-    {
-    cmtk::DebugOutput( 1 ) << "  GE Raw Data Type: " << first->GERawDataType << "\n";
-    }
+			 << "  Description:   " << first->SeriesDescription << "\n"
+			 << "  Series:        " << first->SeriesUID << "\n"
+			 << "  Study:         " << first->StudyUID << "\n"
+			 << "  Acquisition:   " << first->AcquisitionNumber << "\n"
+			 << "  TR / TE:       " << first->RepetitionTime << "ms /" << first->EchoTime << "ms\n"
+			 << "  Position:      " << first->ImagePositionPatient << "\n"
+			 << "  Orientation:   " << first->ImageOrientationPatient << "\n"
+			 << "  Raw Data Type: " << first->RawDataType << "\n";
     
   cmtk::DebugOutput( 1 ) << "\nImage List:\n";
   for ( const_iterator it = this->begin(); it != this->end(); ++it ) 
@@ -485,19 +799,24 @@ VolumeDCM::WriteToArchive( const std::string& fname ) const
     cmtk::DebugOutput( 1 ) << (*it)->fname << " ";
     }
   cmtk::DebugOutput( 1 ) << "\n====================================================\n";
+
+  return volume;
 }
 
+/// Class handling a list of image stacks, i.e., volumes.
 class VolumeList : 
-  public std::vector<VolumeDCM*> 
+  public std::vector<ImageStack*> 
 {
 public:
-  void AddImageFileDCM( ImageFileDCM *const newImage );
+  /// Add a new image file to the correct stack or create a new stack.
+  void AddImageFile( ImageFile *const newImage );
   
-  void WriteToArchive();
+  /// Write all volumes in this list.
+  void WriteVolumes();
 };
 
-inline std::string &
-replacein(std::string &s, const std::string &sub, const std::string &other)
+inline std::string&
+replacein(std::string& s, const std::string& sub, const std::string& other)
 {
   assert(!sub.empty());
   for ( size_t b = s.find(sub, 0); b != s.npos; b = s.find(sub, b) )
@@ -520,13 +839,19 @@ MakeLegalInPath( const std::string& s )
   return result;
 }
 
+std::string
+PutNumberAndSanitize( const std::string& path, const std::string& numberString )
+{
+  std::string result = path;
+  return replacein( replacein( result, "%n", numberString ), "%N", "-" + numberString );
+}
+
 void
-VolumeList::WriteToArchive() 
+VolumeList::WriteVolumes() 
 {
   size_t cntSingleImages = 0;
 
-  int idx = 1;
-  std::map< std::string,std::vector<const VolumeDCM*> > pathToVolumeMap;
+  std::map< std::string,std::vector<const ImageStack*> > pathToVolumeMap;
   for ( const_iterator it = begin(); it != end(); ++it ) 
     {
     if ( ((*it)->size() > 1) || (*(*it)->begin())->IsMultislice )
@@ -536,7 +861,7 @@ VolumeList::WriteToArchive()
       replacein( path, "%D", MakeLegalInPath( (*it)[0][0]->SeriesDescription ) );
       replacein( path, "%R", MakeLegalInPath( (*it)[0][0]->RepetitionTime ) );
       replacein( path, "%E", MakeLegalInPath( (*it)[0][0]->EchoTime ) );
-      replacein( path, "%T", GERawDataTypeString[(*it)[0][0]->GERawDataType] );
+      replacein( path, "%T", (*it)[0][0]->RawDataType );
       
       if ( path.length() > PATH_MAX )
 	cmtk::StdErr << "ERROR: output path exceeds maximum path length";
@@ -557,7 +882,7 @@ VolumeList::WriteToArchive()
     cmtk::DebugOutput( 1 ) << "WARNING: " << cntSingleImages << " single image(s) could not be assigned to multi-image stacks:\n\n";
     for ( const_iterator it = begin(); it != end(); ++it ) 
       {
-      if ( (*it)->size() == 1 )
+      if ( ((*it)->size() == 1) && !(*(*it)->begin())->IsMultislice )
 	{
 	(*(*it))[0]->Print();
 	cmtk::DebugOutput( 1 ) << "\n";
@@ -566,7 +891,7 @@ VolumeList::WriteToArchive()
     cmtk::DebugOutput( 1 ) << "\n====================================================\n";
     }
   
-  for ( std::map< std::string,std::vector<const VolumeDCM*> >::const_iterator it = pathToVolumeMap.begin(); it != pathToVolumeMap.end(); ++it )
+  for ( std::map< std::string,std::vector<const ImageStack*> >::const_iterator it = pathToVolumeMap.begin(); it != pathToVolumeMap.end(); ++it )
     {						
     const size_t nVolumes = it->second.size();
 
@@ -574,13 +899,14 @@ VolumeList::WriteToArchive()
     if ( nVolumes == 1 )
       {					       
       // if there's a "number" tag, get rid of it.
-      std::string uniquePath = it->first;
-      replacein( uniquePath, "%n", "" );
-      replacein( uniquePath, "%N", "" );
+      std::string uniquePath = PutNumberAndSanitize( it->first, "" );
 
-      char finalPath[PATH_MAX];
-      sprintf( finalPath, uniquePath.c_str(), idx++ );
-      it->second[0]->WriteToArchive( finalPath );
+      cmtk::UniformVolume::SmartConstPtr volume = it->second[0]->WriteImage( uniquePath.c_str() );
+
+      if ( WriteXML )
+	{
+	it->second[0]->WriteXML( (uniquePath+".xml").c_str(), *volume );
+	}
       }
     else
       {			
@@ -592,13 +918,14 @@ VolumeList::WriteToArchive()
 	numberString.fill( '0' );
 	numberString << std::right << 1+i;
 
-	std::string uniquePath = it->first;
-	replacein( uniquePath, "%n", numberString.str() );
-	replacein( uniquePath, "%N", "-" + numberString.str() );
+	std::string uniquePath = PutNumberAndSanitize( it->first, numberString.str() );
 
-	char finalPath[PATH_MAX];
-	sprintf( finalPath, uniquePath.c_str(), idx++ );
-	it->second[i]->WriteToArchive( finalPath );
+	cmtk::UniformVolume::SmartConstPtr volume = it->second[i]->WriteImage( uniquePath.c_str() );
+
+	if ( WriteXML )
+	  {
+	  it->second[i]->WriteXML( (uniquePath + ".xml").c_str(), *volume );
+	  }
 	}
       }
     }
@@ -606,23 +933,23 @@ VolumeList::WriteToArchive()
 
 
 void
-VolumeList::AddImageFileDCM( ImageFileDCM *const newImage )
+VolumeList::AddImageFile( ImageFile *const newImage )
 {
   if ( empty() ) 
     {
-    VolumeDCM *newVolumeDCM = new VolumeDCM;
-    newVolumeDCM->AddImageFileDCM( newImage );
-    push_back( newVolumeDCM );
+    ImageStack *newImageStack = new ImageStack;
+    newImageStack->AddImageFile( newImage );
+    push_back( newImageStack );
     } 
   else
     {
     const_iterator it = begin();
     while ( it != end() ) 
       {
-      VolumeDCM *study = *it;
+      ImageStack *study = *it;
       if ( study->Match( newImage ) ) 
 	{
-	study->AddImageFileDCM( newImage );
+	study->AddImageFile( newImage );
 	return;
 	} 
       else 
@@ -630,18 +957,18 @@ VolumeList::AddImageFileDCM( ImageFileDCM *const newImage )
 	++it;
 	}
       }
-    VolumeDCM *newVolumeDCM = new VolumeDCM;
-    newVolumeDCM->AddImageFileDCM( newImage );
-    push_back( newVolumeDCM );    
+    ImageStack *newImageStack = new ImageStack;
+    newImageStack->AddImageFile( newImage );
+    push_back( newImageStack );    
     }
 }
 
 int
-traverse_directory( VolumeList& studylist, const std::string& path, const char *wildcard )
+traverse_directory( VolumeList& volumeList, const std::string& path, const char *wildcard )
 {
   char fullname[PATH_MAX];
 
-  std::vector<ImageFileDCM*> fileList;
+  std::vector<ImageFile*> fileList;
 
 #ifdef _MSC_VER
   WIN32_FIND_DATA fData;
@@ -655,14 +982,14 @@ traverse_directory( VolumeList& studylist, const std::string& path, const char *
       {
       if ( Recursive && (fData.cFileName[0] != '.') )
 	{
-	traverse_directory( studylist, fullname, wildcard );
+	traverse_directory( volumeList, fullname, wildcard );
 	}
       }
     else
       {
       try
 	{
-	fileList.push_back( new ImageFileDCM( fullname ) );
+	fileList.push_back( new ImageFile( fullname ) );
 	(cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
 	}
       catch ( ... )
@@ -687,7 +1014,7 @@ traverse_directory( VolumeList& studylist, const std::string& path, const char *
 	if ( S_ISDIR( entry_status.st_mode ) && Recursive && (entry_pointer->d_name[0] != '.') ) 
 	  {
 	  strcat( fullname, "/" );
-	  traverse_directory( studylist, fullname, wildcard );
+	  traverse_directory( volumeList, fullname, wildcard );
 	  } 
 	else
 	  {
@@ -695,7 +1022,7 @@ traverse_directory( VolumeList& studylist, const std::string& path, const char *
 	    {
 	    try
 	      {
-	      fileList.push_back( new ImageFileDCM( fullname ) );
+	      fileList.push_back( new ImageFile( fullname ) );
 	      (cmtk::StdErr << "\r" << progress_chars[ ++progress % 4 ]).flush();
 	      }
 	    catch ( ... )
@@ -712,22 +1039,22 @@ traverse_directory( VolumeList& studylist, const std::string& path, const char *
 
   switch ( SortFiles )
     {
-    case 0:
+    case SORT_NONE:
     default:
       break;
-    case 1:
-      std::sort( fileList.begin(), fileList.end(), ImageFileDCM::lessFileName );
+    case SORT_FILENAME:
+      std::sort( fileList.begin(), fileList.end(), ImageFile::lessFileName );
       break;
-    case 2:
-      std::sort( fileList.begin(), fileList.end(), ImageFileDCM::lessInstanceNumber );
+    case SORT_INSTANCE_NUMBER:
+      std::sort( fileList.begin(), fileList.end(), ImageFile::lessInstanceNumber );
       break;
     }
   
-  for ( std::vector<ImageFileDCM*>::const_iterator it = fileList.begin(); it != fileList.end(); ++it )
+  for ( std::vector<ImageFile*>::const_iterator it = fileList.begin(); it != fileList.end(); ++it )
     {
     try 
       {
-      studylist.AddImageFileDCM( *it );
+      volumeList.AddImageFile( *it );
       }
     catch (int)
       {
@@ -773,7 +1100,6 @@ doMain ( const int argc, const char *argv[] )
     typedef cmtk::CommandLine::Key Key;
     cl.BeginGroup( "Input", "Input Options");
     cl.AddSwitch( Key( 'r', "recurse" ), &Recursive, true, "Recurse into directories" );
-    cl.AddSwitch( Key( "ge-extensions" ), &WithExtensionsGE, true, "Enable GE extensions (e.g., detect image type magnitude vs. complex)" )->SetProperties( cmtk::CommandLine::PROPS_ADVANCED );
     cl.EndGroup();
 
     cl.BeginGroup( "Output", "Output Options");
@@ -784,7 +1110,9 @@ doMain ( const int argc, const char *argv[] )
 		  "%D (DICOM SeriesDescription); "
 		  "%R (DICOM RepetitionTime - MRI only)"
 		  "%E (DICOM EchoTime - MRI only)"
-		  "%T (GE RawDataType - vendor-specific, MRI only)" );
+		  "%T (RawDataType - vendor-specific, currently GE MRI only)" );
+
+    cl.AddSwitch( Key( 'x', "xml" ), &WriteXML, true, "Write XML sidecar file for each created image." );
 
     cmtk::CommandLine::EnumGroup<EmbedInfoEnum>::SmartPtr embedGroup = cl.AddEnum( "embed", &EmbedInfo, "Embed DICOM information into output images as 'description' (if supported by output file format)." );
     embedGroup->AddSwitch( Key( "StudyID_StudyDate" ), EMBED_STUDYID_STUDYDATE, "StudyID, tag (0020,0010), then underscore, followed by StudyDate, tag (0008,0020). "
@@ -795,9 +1123,9 @@ doMain ( const int argc, const char *argv[] )
     cl.EndGroup();
 
     cl.BeginGroup( "Sorting", "Sorting Options")->SetProperties( cmtk::CommandLine::PROPS_ADVANCED );
-    cl.AddSwitch( Key( "no-sort" ), &SortFiles, 0, "Do NOT sort files by file name (determines order when resolving spatial collisions)" );
-    cl.AddSwitch( Key( "sort-name" ), &SortFiles, 1, "Sort files lexicographically by file name." );
-    cl.AddSwitch( Key( "sort-instance" ), &SortFiles, 2, "Sort files by image instance number. Use this when file names are different lengths, etc." );
+    cl.AddSwitch( Key( "no-sort" ), &SortFiles, SORT_NONE, "Do NOT sort files by file name (sorting determines image stack order when resolving spatial collisions)" );
+    cl.AddSwitch( Key( "sort-by-name" ), &SortFiles, SORT_FILENAME, "Sort files lexicographically by file name. Use this when instance numbers are non-unique." );
+    cl.AddSwitch( Key( "sort-by-instance" ), &SortFiles, SORT_INSTANCE_NUMBER, "Sort files by image instance number. Use this when file names are different lengths, etc." );
     cl.EndGroup();
 
     cl.BeginGroup( "Stacking", "Stacking Options")->SetProperties( cmtk::CommandLine::PROPS_ADVANCED );
@@ -816,20 +1144,20 @@ doMain ( const int argc, const char *argv[] )
     throw cmtk::ExitException( 1 );
     }
   
-  if ( WithExtensionsGE && !dcmDataDict.rdlock().findEntry( "RawDataType_ImageType" ) )
+  if ( !dcmDataDict.rdlock().findEntry( "RawDataType_ImageType" ) )
     {
     dcmDataDict.unlock();
     dcmDataDict.wrlock().addEntry( new DcmDictEntry( 0x0043, 0x102f, EVR_SS, "RawDataType_ImageType", 1, 1, NULL, OFFalse, "GE" ) );
     dcmDataDict.unlock();
     }
-
-  VolumeList studylist;
+  
+  VolumeList volumeList;
   for ( std::vector<std::string>::const_iterator it = SearchRootDirVector.begin(); it != SearchRootDirVector.end(); ++it )
     {
-    traverse_directory( studylist, *it, "*" );
+    traverse_directory( volumeList, *it, "*" );
     }
   
-  studylist.WriteToArchive();
+  volumeList.WriteVolumes();
 
   return 0;
 }
