@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3648 $
+//  $Revision: 3980 $
 //
-//  $LastChangedDate: 2011-12-22 10:58:18 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-03-06 15:54:24 -0800 (Tue, 06 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -352,7 +352,7 @@ CallbackThreshBelow( const char* argv )
 {
   CheckStackOneImage( "ThreshBelow" );
 
-  const float threshold = atof( argv );
+  const cmtk::Types::DataItem threshold = static_cast<cmtk::Types::DataItem>( atof( argv ) );
   
   ImageStackType::iterator it = ImageStack.begin();
   while ( it != ImageStack.end() )
@@ -375,7 +375,7 @@ CallbackThreshAbove( const char* argv )
 {
   CheckStackOneImage( "ThreshAbove" );
 
-  const float threshold = atof( argv );
+  const cmtk::Types::DataItem threshold = static_cast<cmtk::Types::DataItem>( atof( argv ) );
   
   ImageStackType::iterator it = ImageStack.begin();
   while ( it != ImageStack.end() )
@@ -1209,7 +1209,7 @@ CallbackMaxIndex()
   for ( int i = 0; i < static_cast<int>( numberOfPixels ); ++i )
 #endif
     {
-    float maxValue = 0;
+    cmtk::Types::DataItem maxValue = 0;
     short maxIndex = -2;
     cmtk::Types::DataItem v;
     
