@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 4264 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-04-26 11:48:32 -0700 (Thu, 26 Apr 2012) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -49,7 +49,6 @@
 
 #include <IO/cmtkVolumeIO.h>
 #include <IO/cmtkClassStream.h>
-#include <IO/cmtkClassStreamXform.h>
 #include <IO/cmtkClassStreamAffineXform.h>
 
 #include <algorithm>
@@ -146,7 +145,7 @@ GetReconstructedImage( cmtk::UniformVolume::SmartPtr& volume, cmtk::UniformVolum
       cmtk::DebugOutput( 2 ) << "Exporting transformations between passes to " << ExportXformsPath << "\n";
       for ( unsigned int pass = 0; pass < NumberOfPasses; ++pass )
 	{
-	stream << *xformsToPassImages[pass];
+	stream << dynamic_cast<cmtk::AffineXform&>( *xformsToPassImages[pass] );
 	}
       }
     else
