@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3668 $
+//  $Revision: 4111 $
 //
-//  $LastChangedDate: 2011-12-23 11:04:40 -0800 (Fri, 23 Dec 2011) $
+//  $LastChangedDate: 2012-03-30 14:30:21 -0700 (Fri, 30 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -43,7 +43,7 @@
 #include <Base/cmtkUniformVolumeInterpolator.h>
 #include <Base/cmtkLinearInterpolator.h>
 #include <Base/cmtkCubicInterpolator.h>
-#include <Base/cmtkUniformVolumeFilter.h>
+#include <Base/cmtkUniformVolumeGaussianFilter.h>
 
 #include <Registration/cmtkAffineMultiChannelRegistrationFunctional.h>
 #include <Registration/cmtkMultiChannelRMIRegistrationFunctional.h>
@@ -104,7 +104,7 @@ MakeDownsampled( cmtk::UniformVolume::SmartConstPtr& image, const int downsample
   if ( (smoothSigmaFactor > 0) && downsample )
     {
     const cmtk::Units::GaussianSigma sigma( smoothSigmaFactor * downsample * image->GetMinDelta() );
-    result->SetData( cmtk::UniformVolumeFilter( image ).GetDataGaussFiltered( sigma ) );
+    result->SetData( cmtk::UniformVolumeGaussianFilter( image ).GetFiltered3D( sigma ) );
     }
   else
     {
