@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 4126 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-04-04 14:50:32 -0700 (Wed, 04 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -85,11 +85,6 @@ doMain( const int argc, const char* argv[] )
     }
 
   cmtk::UniformVolume::SmartPtr volume( cmtk::VolumeIO::ReadOriented( InputFilePath ) );
-  if ( ! volume || ! volume->GetData() )
-    {
-    cmtk::StdErr << "ERROR: Could not read image " << InputFilePath << "\n";
-    return 1;
-    }
 
   for ( int i = 0; i < Factor; ++i )
     {
