@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2011 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 4250 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-04-24 18:25:08 -0700 (Tue, 24 Apr 2012) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -37,14 +37,11 @@
 #include <System/cmtkCommandLine.h>
 
 #include <IO/cmtkVolumeIO.h>
-#include <IO/cmtkClassStream.h>
 #include <IO/cmtkTypedStreamStudylist.h>
 
 #include <Base/cmtkSplineWarpXform.h>
 #include <Base/cmtkHistogram.h>
 #include <Base/cmtkValueSequence.h>
-#include <Base/cmtkLandmark.h>
-#include <Base/cmtkLandmarkList.h>
 #include <Base/cmtkMathFunctionWrappers.h>
 
 #include <Registration/cmtkReformatVolume.h>
@@ -65,7 +62,6 @@ bool ExpData = false;
 bool WriteAsColumn = false;
 bool OutputExpNotation = false;
 
-const char *LandmarksFileName = NULL;
 const char *MaskFileName = NULL;
 bool MaskIsBinary = false;
 int MaskOutputAllUpTo = 0;
@@ -141,8 +137,6 @@ AnalyzeLabels( const cmtk::UniformVolume* volume, const cmtk::TypedArray* maskDa
       }
     }
   
-  cmtk::LandmarkList landmarkList;
-  
   cmtk::DebugOutput( 1 ) << "idx\t\tcount\t\tsurface\t\tvolume\tCenterOfMass\n";
 
   const cmtk::Types::Coordinate voxelVolume = volume->m_Delta[0] * volume->m_Delta[1] * volume->m_Delta[2];
@@ -162,23 +156,9 @@ AnalyzeLabels( const cmtk::UniformVolume* volume, const cmtk::TypedArray* maskDa
 	fprintf( stdout, "%03d\t%14d\t%14d\t%.4f\t(%f,%f,%f)\n", 
 		 (int)(idx+range.m_LowerBound), count[idx], countSurface[idx], count[idx] * voxelVolume, centerOfMass[idx][0], centerOfMass[idx][1], centerOfMass[idx][2] );
       totalCount += count[idx];
-      
-      cmtk::Landmark* landmark = new cmtk::Landmark();
-      char name[4];
-      sprintf( name, "%3d", idx );
-      landmark->SetName( name );
-      landmark->SetLocation( centerOfMass[idx].begin() );
-      landmarkList.push_back( cmtk::Landmark::SmartPtr( landmark ) );
       }
     }
   
-  if ( LandmarksFileName ) 
-    {
-    cmtk::ClassStream stream( LandmarksFileName, cmtk::ClassStream::MODE_WRITE );
-    stream << landmarkList;
-    stream.Close();
-    }
-  
   cmtk::Types::DataItem entropy = 0;
   if ( totalCount )
     {
@@ -351,7 +331,6 @@ doMain ( const int argc, const char* argv[] )
     
     cl.AddSwitch( Key( 'C', "column" ), &WriteAsColumn, true, "Write statistics in column format rather than line format." );
     cl.AddSwitch( Key( 'E', "e-notation" ), &OutputExpNotation, true, "Write floating point numbers in #e# notation (i.e., 1e-3 instead of 0.001)" ); 
-    cl.AddOption( Key( 'c', "com" ), &LandmarksFileName, "Write centers of mass of labeled regions to landmarks file." );
     
     cl.AddOption( Key( 'm', "mask" ), &MaskFileName, "Analyze region based on binary mask file", &MaskIsBinary );
     cl.AddOption( Key( 'M', "Mask" ), &MaskFileName, "Analyze regions separately based on mask file (label field)." );
