@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3499 $
+//  $Revision: 4236 $
 //
-//  $LastChangedDate: 2011-10-21 12:48:54 -0700 (Fri, 21 Oct 2011) $
+//  $LastChangedDate: 2012-04-20 14:55:02 -0700 (Fri, 20 Apr 2012) $
 //
-//  $LastChangedBy: torsten_at_home $
+//  $LastChangedBy: torstenrohlfing $
 //
 */
 
@@ -58,9 +58,7 @@ AffineXform::MakeIdentityXform ()
 
 AffineXform::AffineXform ( const AffineXform& other ) :
   Xform( other ),
-  Matrix( NULL ), 
-  m_LogScaleFactors( false ),
-  InverseXform( NULL )
+  m_LogScaleFactors( false )
 { 
   this->AllocateParameterVector( TotalNumberOfParameters );
   (*this->m_ParameterVector) = (*other.m_ParameterVector);
@@ -513,19 +511,4 @@ AffineXform::CanonicalRotationRange()
     }
 }
 
-void
-AffineXform::Print() const
-{
-  StdErr.printf( "AffineXform at %p:\n", this );
-  StdErr.printf( "\tNumber DOFs: %d\n", NumberDOFs );
-  StdErr.printf( "\tTranslation: [%f,%f,%f]\n", this->m_Parameters[0], this->m_Parameters[1], this->m_Parameters[2] );
-  StdErr.printf( "\tRotation: [%f,%f,%f] around [%f,%f,%f]\n", 
-		    this->m_Parameters[3], this->m_Parameters[4], this->m_Parameters[5], 
-		    this->m_Parameters[12], this->m_Parameters[13], this->m_Parameters[14] );
-  StdErr.printf( "\tScale: [%f,%f,%f]\n", this->m_Parameters[6], this->m_Parameters[7], this->m_Parameters[8] );
-  StdErr.printf( "\tShear: [%f,%f,%f]\n", this->m_Parameters[9], this->m_Parameters[10], this->m_Parameters[11] );
-
-  this->Matrix.Print( StdErr );
-}
-
 } // namespace
