@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3878 $
+//  $Revision: 4230 $
 //
-//  $LastChangedDate: 2012-02-15 15:55:31 -0800 (Wed, 15 Feb 2012) $
+//  $LastChangedDate: 2012-04-19 14:27:53 -0700 (Thu, 19 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -493,9 +493,6 @@ public:
   /// Assignment operator.
   AffineXform& operator=( const AffineXform& other );
 
-  /// Print object.
-  virtual void Print() const;
-
 protected:
   /// Clone this object.
   virtual Self* CloneVirtual() const
