@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3642 $
+//  $Revision: 4137 $
 //
-//  $LastChangedDate: 2011-12-21 14:50:10 -0800 (Wed, 21 Dec 2011) $
+//  $LastChangedDate: 2012-04-06 10:01:14 -0700 (Fri, 06 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -210,24 +210,34 @@ public:
     x = (offset % nextK) % nextJ;
   }
 
+  /// Get index of a pixel identified by its offset.
+  Self::IndexType GetIndexFromOffset( const size_t offset ) const 
+  {
+    Self::IndexType index;
+    index[2] = offset / nextK;
+    index[1] = (offset % nextK) / nextJ;
+    index[0] = (offset % nextK) % nextJ;
+    return index;
+  }
+
   /// Return data at specified offset
   bool GetDataAt ( Types::DataItem& data, const size_t offset ) const 
   {
     return this->m_Data->Get( data, offset );
   }
 
+  /// Return data at specified grid point.
+  bool GetDataAt ( Types::DataItem& data, const int x, const int y, const int z ) const
+  {
+    return this->m_Data->Get( data, this->GetOffsetFromIndex( x, y, z ) );
+  }
+
   /// Set data at specified offset
   void SetDataAt ( const Types::DataItem data, const size_t offset )
   {
     this->m_Data->Set( data, offset );
   }
   
-  /// Return data at specified grid point.
-  bool GetDataAt ( Types::DataItem& data, const int x, const int y, const int z ) const
-  {
-    return this->GetDataAt( data, this->GetOffsetFromIndex( x, y, z ) );
-  }
-
   /// Set data at specified grid point.
   void SetDataAt ( const Types::DataItem data, const int x, const int y, const int z )
   {
@@ -247,11 +257,7 @@ public:
   /// Return data at specified grid offset, or a given default value if no data exists there.
   Types::DataItem GetDataAt ( const size_t offset, const Types::DataItem defaultValue = 0.0 ) const
   {
-    Types::DataItem value;
-    if ( this->GetDataAt( value, offset ) )
-      return value;
-    else
-      return defaultValue;
+    return this->m_Data->ValueAt( offset, defaultValue );
   }
 
   /** Return data after mirroring.
