@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3861 $
+//  $Revision: 4236 $
 //
-//  $LastChangedDate: 2012-02-14 12:56:27 -0800 (Tue, 14 Feb 2012) $
+//  $LastChangedDate: 2012-04-20 14:55:02 -0700 (Fri, 20 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -147,14 +147,12 @@ public:
   virtual void GetTransformedGridRow( Self::SpaceVectorType *const v, const int numPoints, const int idxX, const int idxY, const int idxZ ) const;
   
   /// Get Jacobian matrix.
-  virtual void GetJacobian( const Self::SpaceVectorType& v, CoordinateMatrix3x3& J ) const;
+  virtual CoordinateMatrix3x3 GetJacobian( const Self::SpaceVectorType& v ) const;
 
   /// Compute Jacobian determinant at a certain location.
   virtual Types::Coordinate GetJacobianDeterminant ( const Self::SpaceVectorType& v ) const
   {
-    CoordinateMatrix3x3 J;
-    this->GetJacobian( v, J );
-    return J.Determinant();
+    return this->GetJacobian( v ).Determinant();
   }
   
   /// Return 1.0 since deformation field DOFs are always direct deformations in space units.
