@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3798 $
+//  $Revision: 4308 $
 //
-//  $LastChangedDate: 2012-02-02 14:44:12 -0800 (Thu, 02 Feb 2012) $
+//  $LastChangedDate: 2012-05-04 10:33:11 -0700 (Fri, 04 May 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -35,6 +35,8 @@
 
 #include <cmtkconfig.h>
 
+#include <Base/cmtkDataTypeTraits.h>
+
 #include <System/cmtkSmartPtr.h>
 #include <System/cmtkSmartConstPtr.h>
 
@@ -46,7 +48,7 @@
 namespace
 cmtk
 {
-/// Class for n-dimensional image index.
+/// Class for fixed-size n-dimensional vector.
 template<size_t NDIM,typename T=int>
 class FixedVector
 {
@@ -63,11 +65,14 @@ public:
   /// Smart pointer-to-const to this class.
   typedef SmartConstPointer<Self> SmartConstPtr;
 
+  /// Return vector size.
+  size_t Size() const { return NDIM; }
+
   /// Zero operator.
   static const Self Zero()
   {
     Self v;
-    std::fill( v.begin(), v.end(), 0 );
+    std::fill( v.begin(), v.end(), DataTypeTraits<T>::Zero() );
     return v;
   }
 
@@ -104,7 +109,7 @@ public:
   FixedVector( const FixedVector<NDIM,T2>& rhs )
   {
     for ( size_t i = 0; i < NDIM; ++i )
-      this->m_Data[i] = rhs[i];
+      this->m_Data[i] = static_cast<T>( rhs[i] );
   }
 
   /// Get element reference.
@@ -349,7 +354,7 @@ operator*( const T2 lhs, const FixedVector<NDIM,T>& rhs )
 {
   FixedVector<NDIM,T> result( rhs );
   for ( size_t i = 0; i<NDIM; ++i )
-    result[i] *= lhs;
+    result[i] *= static_cast<T>( lhs );
   return result;
 }
 
@@ -375,7 +380,7 @@ Min( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
   return result;
 }
 
-/// Less-than operator.
+/// Element-wise less-than operator.
 template<size_t NDIM,typename T>
 bool
 operator<( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
@@ -386,7 +391,18 @@ operator<( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
   return true;
 }
 
-/// Stream input operator.
+/// Element-wise less-than-or-equal operator.
+template<size_t NDIM,typename T>
+bool
+operator<=( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
+{
+  for ( size_t i = 0; i < NDIM; ++i )
+    if ( lhs[i] > rhs[i] )
+      return false;
+  return true;
+}
+
+/// Stream output operator.
 template<size_t NDIM,typename T>
 std::ostream& operator<<( std::ostream& stream, const FixedVector<NDIM,T>& index )
 {
@@ -396,7 +412,7 @@ std::ostream& operator<<( std::ostream& stream, const FixedVector<NDIM,T>& index
   return stream;
 }
 
-/// Stream output operator.
+/// Stream input operator.
 template<size_t NDIM,typename T>
 std::istream& operator>>( std::istream& stream, FixedVector<NDIM,T>& index )
 {
