@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3517 $
+//  $Revision: 4230 $
 //
-//  $LastChangedDate: 2011-10-27 12:42:15 -0700 (Thu, 27 Oct 2011) $
+//  $LastChangedDate: 2012-04-19 14:27:53 -0700 (Thu, 19 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -157,9 +157,9 @@ ScalarImage::ScalarImage
 
 ScalarImage*
 ScalarImage::InterpolateFrom
-( const ScalarImage* grid, const CoordinateMatrix3x3* matrix, const cmtk::Interpolators::InterpolationEnum interpolation ) const
+( const ScalarImage& grid, const CoordinateMatrix3x3& matrix, const cmtk::Interpolators::InterpolationEnum interpolation ) const
 {
-  int dimsX = grid->m_Dims[0], dimsY = grid->m_Dims[1];
+  int dimsX = grid.m_Dims[0], dimsY = grid.m_Dims[1];
   ScalarImage* result = new ScalarImage( dimsX, dimsY );
 
   // if we don;t have pixel data, don't interpolate
@@ -174,24 +174,24 @@ ScalarImage::InterpolateFrom
   origin[0] = origin[1] = 0;
 
   FixedVector<2,Types::Coordinate> deltax;
-  deltax[0] = grid->m_PixelSize[0];
+  deltax[0] = grid.m_PixelSize[0];
   deltax[1] = 0;
-
+  
   FixedVector<2,Types::Coordinate> deltay;
   deltay[0] = 0;
-  deltay[1] = grid->m_PixelSize[1];
+  deltay[1] = grid.m_PixelSize[1];
 
-  matrix->Multiply( origin );
+  origin *= matrix;
   origin[0] /= this->m_PixelSize[0];
   origin[1] /= this->m_PixelSize[1];
 
-  matrix->Multiply( deltax );
+  deltax *= matrix;
   deltax[0] /= this->m_PixelSize[0];
   deltax[1] /= this->m_PixelSize[1];
   deltax[0] -= origin[0];
   deltax[1] -= origin[1];
 
-  matrix->Multiply( deltay );
+  deltay *= matrix;
   deltay[0] /= this->m_PixelSize[0];
   deltay[1] /= this->m_PixelSize[1];
   deltay[0] -= origin[0];
