@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3871 $
+//  $Revision: 4275 $
 //
-//  $LastChangedDate: 2012-02-15 14:39:46 -0800 (Wed, 15 Feb 2012) $
+//  $LastChangedDate: 2012-04-29 12:39:40 -0700 (Sun, 29 Apr 2012) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -166,19 +166,6 @@ public:
   /// Return rigidity constraint of the current transformation grid with local weights.
   virtual Types::Coordinate GetRigidityConstraint( const DataGrid* weightMap ) const;
 
-  /** Return sparse Jacobian constraint of the current transformation grid.
-   * Unlike GetJacobianConstraint(), this function does not evaluate the
-   * Jacobian constraint at each voxel of the registered reference image; it
-   * merely performs the evaluation at the control points of the deformation,
-   * thereby greatly increasing computational performance at the cost of less
-   * accurate and stable computation.
-   */
-  virtual Types::Coordinate GetJacobianConstraintSparse() const;
-
-  /** Return sparse rigidity constraint of the current transformation grid.
-   */
-  virtual Types::Coordinate GetRigidityConstraintSparse() const;
-
   /// Return derivative of Jacobian constraint with respect to one parameter.
   virtual void GetJacobianConstraintDerivative( double& lower, double& upper, const int param, const UniformVolume::RegionType&, const Types::Coordinate step ) const;
 
@@ -238,22 +225,20 @@ public:
   /// Replace existing vector with transformed location.
   virtual void ApplyInPlace( Self::SpaceVectorType& v ) const 
   {
-    Types::Coordinate r[3], f[3];
+    Types::Coordinate f[3];
     int grid[3];
     
     // Do some precomputations.
-    { 
     for ( int dim = 0; dim<3; ++dim ) 
       {
       // This is the (real-valued) index of the control point grid cell the
       // given location is in.
-      r[dim] = this->m_InverseSpacing[dim] * v[dim];
+      const Types::Coordinate r = this->m_InverseSpacing[dim] * v[dim];
       // This is the actual cell index.
-      grid[dim] = std::min<int>( static_cast<int>( r[dim] ), this->m_Dims[dim]-4 );
+      grid[dim] = std::min<int>( static_cast<int>( r ), this->m_Dims[dim]-4 );
       // And here's the relative position within the cell.
-      f[dim] = r[dim] - grid[dim];
+      f[dim] = r - grid[dim];
       }
-    }
 
     // Create a pointer to the front-lower-left corner of the c.p.g. cell.
     const Types::Coordinate* coeff = this->m_Parameters + 3 * ( grid[0] + this->m_Dims[0] * (grid[1] + this->m_Dims[1] * grid[2]) );
@@ -290,6 +275,25 @@ public:
       ++coeff;
       }
   }
+  
+  /// Precompute spline parameters for one point.
+  void PrecomputeLocationSpline( const Self::SpaceVectorType& v, FixedVector<3,int>& grid, FixedVector< 3, FixedVector<4,Types::Coordinate> >& spline  ) const
+  {
+    for ( int dim = 0; dim<3; ++dim ) 
+      {
+      // This is the (real-valued) index of the control point grid cell the given location is in.
+      const Types::Coordinate r = this->m_InverseSpacing[dim] * v[dim];
+      // This is the actual cell index.
+      grid[dim] = std::min<int>( static_cast<int>( r ), this->m_Dims[dim]-4 );
+      // And here's the relative position within the cell.
+      const Types::Coordinate fractional = r - grid[dim];
+      for ( int k = 0; k < 4; ++k )
+	{
+	spline[dim][k] = CubicSpline::ApproxSpline( k, fractional );
+	}
+      }
+  }
+
 
   /// Get volume influenced by one parameter.
   virtual UniformVolume::CoordinateRegionType GetVolumeOfInfluence( const size_t idx /*!< Parameter point index */, 
@@ -353,11 +357,8 @@ public:
   /// Get local Jacobian.
   virtual CoordinateMatrix3x3 GetJacobian( const Self::SpaceVectorType& v ) const;
 
-  /// Get local Jacobian into existing matrix.
-  virtual void GetJacobian( const Self::SpaceVectorType& v, CoordinateMatrix3x3& J ) const;
-
   /// Get local Jacobian at control point into existing matrix.
-  virtual void GetJacobianAtControlPoint( const Types::Coordinate* cp, CoordinateMatrix3x3& J ) const;
+  virtual CoordinateMatrix3x3 GetJacobianAtControlPoint( const Types::Coordinate* cp ) const;
 
   /// Get sequence of Jacobians for pixel row.
   virtual void GetJacobianRow( CoordinateMatrix3x3 *const array, const int x, const int y, const int z, const size_t numberOfPoints ) const;
@@ -447,6 +448,9 @@ protected:
 
   /// Fitting class is a friend.
   friend class FitSplineWarpToDeformationField;
+
+  /// Fitting class is a friend.
+  friend class FitSplineWarpToLandmarks;
 };
 
 } // namespace
