@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3765 $
+//  $Revision: 4275 $
 //
-//  $LastChangedDate: 2012-01-31 13:49:41 -0800 (Tue, 31 Jan 2012) $
+//  $LastChangedDate: 2012-04-29 12:39:40 -0700 (Sun, 29 Apr 2012) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -46,14 +46,6 @@ cmtk
 /** \addtogroup Base */
 //@{
 
-CoordinateMatrix3x3
-SplineWarpXform::GetJacobian( const Self::SpaceVectorType& v ) const
-{
-  CoordinateMatrix3x3 J;
-  this->GetJacobian( v, J );
-  return J;
-}
-
 void
 SplineWarpXform::GetJacobianRow
 ( CoordinateMatrix3x3 *const array, const int x, const int y, const int z, const size_t numberOfPoints ) 
@@ -165,11 +157,11 @@ SplineWarpXform::GetJacobianRow
   }
 }
 
-void
+CoordinateMatrix3x3
 SplineWarpXform::GetJacobianAtControlPoint
-( const Types::Coordinate* cp, CoordinateMatrix3x3& J ) const
+( const Types::Coordinate* cp ) const
 {
-  J.Fill( 0.0 );
+  CoordinateMatrix3x3 J = CoordinateMatrix3x3::Zero();
   
   const double  sp[3] = {  1.0/6, 2.0/3, 1.0/6 };
   const double dsp[3] = { -1.0/2,     0, 1.0/2 };
@@ -205,11 +197,13 @@ SplineWarpXform::GetJacobianAtControlPoint
     for ( int j = 0; j<3; ++j )
       J[i][j] *= this->m_InverseSpacing[i];
     }
+
+  return J;
 }
 
-void
+CoordinateMatrix3x3
 SplineWarpXform::GetJacobian
-( const Self::SpaceVectorType& v, CoordinateMatrix3x3& J ) const
+( const Self::SpaceVectorType& v ) const
 {
   Types::Coordinate r[3], f[3];
   int grid[3];
@@ -221,6 +215,7 @@ SplineWarpXform::GetJacobian
     f[dim] = std::max<Types::Coordinate>( 0, std::min<Types::Coordinate>( 1.0, r[dim] - grid[dim] ) );
     }
 
+  CoordinateMatrix3x3 J = CoordinateMatrix3x3::Zero();
   const Types::Coordinate* coeff = this->m_Parameters + 3 * ( grid[0] + this->m_Dims[0] * (grid[1] + this->m_Dims[1] * grid[2]) );
   
   // loop over the three components of the coordinate transformation function,
@@ -273,6 +268,8 @@ SplineWarpXform::GetJacobian
     for ( int j = 0; j<3; ++j )
       J[i][j] *= this->m_InverseSpacing[i];
     }
+
+  return J;
 }
 
 Types::Coordinate
@@ -497,8 +494,7 @@ SplineWarpXform::GetJacobianDeterminantRow
 Types::Coordinate
 SplineWarpXform::JacobianDeterminant ( const Types::Coordinate *cp ) const
 {
-  CoordinateMatrix3x3 J;
-  this->GetJacobianAtControlPoint( cp, J );
+  const CoordinateMatrix3x3 J = this->GetJacobianAtControlPoint( cp );
   
   return this->m_InverseSpacing[0] * this->m_InverseSpacing[1] * this->m_InverseSpacing[2] * 
     ( J[0][0] * (J[1][1]*J[2][2] - J[1][2]*J[2][1]) + 
@@ -570,22 +566,6 @@ SplineWarpXform::GetJacobianConstraint () const
   return constraint / ( VolumeDims[0] * VolumeDims[1] * VolumeDims[2] );
 }
 
-Types::Coordinate
-SplineWarpXform::GetJacobianConstraintSparse () const
-{
-  double Constraint = 0;
-
-  const Types::Coordinate* coeff = this->m_Parameters + nextI + nextJ + nextK;
-  for ( int z = 1; z<this->m_Dims[2]-1; ++z, coeff+=2*nextJ )
-    for ( int y = 1; y<this->m_Dims[1]-1; ++y, coeff+=2*nextI )
-      for ( int x = 1; x<this->m_Dims[0]-1; ++x, coeff+=nextI )
-	Constraint += fabs( log ( this->GetJacobianDeterminant( Self::SpaceVectorType( coeff ) ) / this->m_GlobalScaling ) );
-  
-  // Divide by number of control points to normalize with respect to the
-  // number of local Jacobians in the computation.
-  return (double)(Constraint / this->m_NumberOfControlPoints);
-}
-
 void 
 SplineWarpXform::GetJacobianConstraintDerivative
 ( double& lower, double& upper, const int param, const DataGrid::RegionType& voi, const Types::Coordinate step ) const
