@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3743 $
+//  $Revision: 4236 $
 //
-//  $LastChangedDate: 2012-01-25 15:40:11 -0800 (Wed, 25 Jan 2012) $
+//  $LastChangedDate: 2012-04-20 14:55:02 -0700 (Fri, 20 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -151,7 +151,7 @@ public:
    */
   virtual void CreateDefaultIndexToPhysicalMatrix();
 
-  /** Get matrix that maps form image coordinates to physical space.
+  /** Get matrix that maps from current image coordinates to physical space.
    * The returned matrix is computed by removing pixel size from the one stored
    * in m_IndexToPhysicalMatrix.
    */
@@ -306,7 +306,7 @@ public:
    *
    *\warning No parameter range checking is currently performed!
    */
-  virtual void GetHessianAt( Matrix3x3<Types::DataItem>& H, const int i, const int j, const int k );
+  virtual Matrix3x3<Types::DataItem> GetHessianAt( const int i, const int j, const int k );
 
   /// Get data value at specified coordinate.
   template<class TData> inline bool ProbeData( TData& result, const TData* dataPtr, const Self::CoordinateVectorType& location ) const;
@@ -390,12 +390,12 @@ public:
   /** Get grid index corresponding (as close as possible) to coordinate.
    *\return True if given point is inside image, false if outside.
    */
-  virtual bool GetClosestGridPointIndex( const Self::CoordinateVectorType v, int *const xyz ) const 
+  virtual bool GetClosestGridPointIndex( const Self::CoordinateVectorType v, Self::IndexType& idx ) const 
   {
     for ( int dim = 0; dim < 3; ++dim )
       {
-      xyz[dim] = (int) MathUtil::Round((v[dim]-this->m_Offset[dim]) / this->m_Delta[dim]);
-      if ( (xyz[dim] < 0) || ( xyz[dim] > this->m_Dims[dim]-1) )
+      idx[dim] = static_cast<int>( MathUtil::Round((v[dim]-this->m_Offset[dim]) / this->m_Delta[dim]) );
+      if ( (idx[dim] < 0) || ( idx[dim] > this->m_Dims[dim]-1) )
 	return false;
       }
     return true;
@@ -415,12 +415,12 @@ public:
    *\return True if given point is inside image, false if outside.
    */
   virtual bool GetTruncGridPointIndex( const Self::CoordinateVectorType v /*!< Location to find in the grid. */, 
-				       int *const xyz /*!< Truncated grid point index (i.e., nearest grid point between location and grid coordinate origin. */ ) const 
+				       Self::IndexType& idx /*!< Truncated grid point index (i.e., nearest grid point between location and grid coordinate origin. */ ) const 
   {
     for ( int dim = 0; dim < 3; ++dim )
       {
-      xyz[dim] = static_cast<int>((v[dim]-this->m_Offset[dim]) / this->m_Delta[dim]);
-      if ( (xyz[dim] < 0) || ( xyz[dim] > this->m_Dims[dim]-1) )
+      idx[dim] = static_cast<int>((v[dim]-this->m_Offset[dim]) / this->m_Delta[dim]);
+      if (  (idx[dim] < 0) || (idx[dim] > this->m_Dims[dim]-1) )
 	return false;
       }
     return true;
@@ -437,6 +437,14 @@ public:
     return Self::CoordinateVectorType( loc );
   }
   
+  /** Get a grid location in image coordinates.
+   *\return The location in image coordinates of the given grid element as a Self::CoordinateVectorType.
+   */
+  virtual const Self::CoordinateVectorType GetGridLocation( const Self::CoordinateVectorType& xyz /*!< Grid index (possibly fractional) */ ) const 
+  {
+    return this->m_Offset + ComponentMultiply( xyz, this->m_Delta );
+  }
+  
   /** Get a grid location in physical coordinates.
    *\param idxV The index of the intended grid element with respect to the
    * three coordinate axes. Valid range is from 0 to Dims[...]-1. Fractional coordinates are permitted.
@@ -496,10 +504,14 @@ public:
     return 0.5 * ( region.From() + region.To() );
   }
   
-  /** Return cropped uniform volume.
+  /** Return cropped uniform volume with currently set crop region.
    */
   Self::SmartPtr GetCroppedVolume() const;
 
+  /** Return cropped uniform volume with explicit crop region.
+   */
+  Self::SmartPtr GetCroppedVolume( const Self::RegionType& region ) const;
+
   /** Return projection (e.g., MIP, sum) along one axis.
    * This function calls its equivalent in DataGrid and adds calibration
    * info (i.e., correct pixel sizes) to the resulting image.
