@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3857 $
+//  $Revision: 4234 $
 //
-//  $LastChangedDate: 2012-02-14 11:18:54 -0800 (Tue, 14 Feb 2012) $
+//  $LastChangedDate: 2012-04-19 14:59:41 -0700 (Thu, 19 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -36,6 +36,8 @@ void
 cmtk::UniformVolumePainter::DrawSphere
 ( const UniformVolume::CoordinateVectorType& center, const Types::Coordinate radius, const Types::DataItem value )
 {
+  UniformVolume& volume = *(this->m_Volume);
+
   UniformVolume::CoordinateVectorType centerAbsolute( center );
   Types::Coordinate radiusAbsolute[3] = { radius, radius, radius };
   
@@ -48,29 +50,37 @@ cmtk::UniformVolumePainter::DrawSphere
     case Self::COORDINATES_RELATIVE:
       for ( int dim = 0; dim < 3; ++dim )
 	{
-	centerAbsolute[dim] *= this->m_Volume->Size[dim];
-	radiusAbsolute[dim] *= this->m_Volume->Size[dim];
+	(centerAbsolute[dim] *= volume.Size[dim]) += volume.m_Offset[dim];
+	radiusAbsolute[dim] *= volume.Size[dim];
 	}
       break;
     case Self::COORDINATES_INDEXED:
       for ( int dim = 0; dim < 3; ++dim )
 	{
-	centerAbsolute[dim] *= this->m_Volume->m_Delta[dim];
-	radiusAbsolute[dim] *= this->m_Volume->m_Delta[dim];
+	(centerAbsolute[dim] *= volume.m_Delta[dim]) += volume.m_Offset[dim];
+	radiusAbsolute[dim] *= volume.m_Delta[dim];
 	}
       break;
     }
   
-  size_t offset = 0;
-  for ( int k = 0; k < this->m_Volume->m_Dims[2]; ++k )
+  DataGrid::RegionType region = volume.GetWholeImageRegion();
+  for ( int dim = 0; dim < 3; ++dim )
     {
-    const Types::Coordinate Z = this->m_Volume->GetPlaneCoord( 2, k );
-    for ( int j = 0; j < this->m_Volume->m_Dims[1]; ++j )
+    region.From()[dim] = std::max( region.From()[dim], static_cast<int>( (centerAbsolute[dim] - radiusAbsolute[dim] - volume.m_Offset[dim]) / volume.m_Delta[dim] ) - 1 );
+    region.To()[dim] = std::min( region.To()[dim], static_cast<int>( (centerAbsolute[dim] + radiusAbsolute[dim] - volume.m_Offset[dim]) / volume.m_Delta[dim] ) + 1 );
+    }
+
+  for ( int k = region.From()[2]; k < region.To()[2]; ++k )
+    {
+    const Types::Coordinate Z = volume.GetPlaneCoord( 2, k );
+    for ( int j = region.From()[1]; j < region.To()[1]; ++j )
       {
-    const Types::Coordinate Y = this->m_Volume->GetPlaneCoord( 1, j );
-      for ( int i = 0; i < this->m_Volume->m_Dims[0]; ++i, ++offset )
+      const Types::Coordinate Y = volume.GetPlaneCoord( 1, j );
+      
+      size_t offset = region.From()[0] + volume.m_Dims[0] * ( j + volume.m_Dims[1] * k );
+      for ( int i = region.From()[0]; i < region.To()[0]; ++i, ++offset )
 	{
-	const Types::Coordinate X = this->m_Volume->GetPlaneCoord( 0, i );
+	const Types::Coordinate X = volume.GetPlaneCoord( 0, i );
 	
 	UniformVolume::CoordinateVectorType v = FixedVectorStaticInitializer<3,Types::Coordinate>::Init( X, Y, Z );
 	v -= centerAbsolute;
@@ -81,7 +91,7 @@ cmtk::UniformVolumePainter::DrawSphere
 	  }
 
 	if ( v.RootSumOfSquares() <= 1.0 )
-	  this->m_Volume->SetDataAt( value, offset );
+	  volume.SetDataAt( value, offset );
 	}
       }
     }
@@ -91,6 +101,8 @@ void
 cmtk::UniformVolumePainter::DrawBox
 ( const UniformVolume::CoordinateVectorType& boxFrom, const UniformVolume::CoordinateVectorType& boxTo, const Types::DataItem value )
 {
+  UniformVolume& volume = *(this->m_Volume);
+
   int indexFrom[3], indexTo[3];
 
   switch ( this->m_CoordinateMode )
@@ -99,15 +111,15 @@ cmtk::UniformVolumePainter::DrawBox
     case Self::COORDINATES_ABSOLUTE:
       for ( int dim = 0; dim < 3; ++dim )
 	{
-	indexFrom[dim] = static_cast<int>( MathUtil::Round( boxFrom[dim] / this->m_Volume->m_Delta[dim] ) );
-	indexTo[dim] = static_cast<int>( MathUtil::Round( boxTo[dim] / this->m_Volume->m_Delta[dim] ) );
+	indexFrom[dim] = static_cast<int>( MathUtil::Round( boxFrom[dim] / volume.m_Delta[dim] ) );
+	indexTo[dim] = static_cast<int>( MathUtil::Round( boxTo[dim] / volume.m_Delta[dim] ) );
 	}
       break;
     case Self::COORDINATES_RELATIVE:
       for ( int dim = 0; dim < 3; ++dim )
 	{
-	indexFrom[dim] = static_cast<int>( MathUtil::Round( boxFrom[dim] * this->m_Volume->Size[dim] / this->m_Volume->m_Delta[dim] ) );
-	indexTo[dim] = static_cast<int>( MathUtil::Round( boxTo[dim] * this->m_Volume->Size[dim] / this->m_Volume->m_Delta[dim] ) );
+	indexFrom[dim] = static_cast<int>( MathUtil::Round( boxFrom[dim] * volume.Size[dim] / volume.m_Delta[dim] ) );
+	indexTo[dim] = static_cast<int>( MathUtil::Round( boxTo[dim] * volume.Size[dim] / volume.m_Delta[dim] ) );
 	}
       break;
     case Self::COORDINATES_INDEXED:
@@ -126,8 +138,8 @@ cmtk::UniformVolumePainter::DrawBox
     if ( indexFrom[dim] > indexTo[dim] )
       std::swap( indexFrom[dim], indexTo[dim] );
 
-    indexFrom[dim] = std::max( 0, std::min( this->m_Volume->m_Dims[dim]-1, indexFrom[dim] ) );
-    indexTo[dim] = std::max( 0, std::min( this->m_Volume->m_Dims[dim]-1, indexTo[dim] ) );
+    indexFrom[dim] = std::max( 0, std::min( volume.m_Dims[dim]-1, indexFrom[dim] ) );
+    indexTo[dim] = std::max( 0, std::min( volume.m_Dims[dim]-1, indexTo[dim] ) );
     }
   
   for ( int k = indexFrom[2]; k <= indexTo[2]; ++k )
@@ -136,7 +148,7 @@ cmtk::UniformVolumePainter::DrawBox
       {
       for ( int i = indexFrom[0]; i <= indexTo[0]; ++i )
 	{
-	this->m_Volume->SetDataAt( value, this->m_Volume->GetOffsetFromIndex( i, j, k ) );
+	volume.SetDataAt( value, volume.GetOffsetFromIndex( i, j, k ) );
 	}
       }
     }
