@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3866 $
+//  $Revision: 4093 $
 //
-//  $LastChangedDate: 2012-02-15 10:58:02 -0800 (Wed, 15 Feb 2012) $
+//  $LastChangedDate: 2012-03-27 13:05:25 -0700 (Tue, 27 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -47,7 +47,7 @@
 #include <Base/cmtkAffineXform.h>
 #include <Base/cmtkBitVector.h>
 
-#include <Base/cmtkMatchedLandmarkList.h>
+#include <Base/cmtkLandmarkPairList.h>
 
 #include <System/cmtkSmartPtr.h>
 
@@ -225,7 +225,7 @@ public:
   
   /** Return derivative of registration error with respect to one parameter.
    */
-  virtual void GetDerivativeLandmarksMSD( double& lowerMSD, double& upperMSD, const MatchedLandmarkList* ll, const unsigned int idx, const Types::Coordinate step );
+  virtual void GetDerivativeLandmarksMSD( double& lowerMSD, double& upperMSD, const LandmarkPairList& ll, const unsigned int idx, const Types::Coordinate step );
 
   /** Return inverse consistency.
    */
