@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -42,8 +42,8 @@
 #include <Base/cmtkWarpXform.h>
 #include <Base/cmtkSplineWarpXform.h>
 #include <Base/cmtkParametricPlane.h>
-#include <Base/cmtkLandmark.h>
-#include <Base/cmtkLandmarkList.h>
+
+#include <string>
 
 namespace
 cmtk
@@ -74,7 +74,7 @@ public:
    *\param filename Name of the archive to open.
    *\param mode Access mode, ie. read-only, write-only, etc.
    */
-  ClassStream( const char *filename, const Self::Mode mode )
+  ClassStream( const std::string& filename, const Self::Mode mode )
     : TypedStream( filename,  mode ) {}
 
   /** Open constructor for separate path and archive names.
@@ -82,18 +82,20 @@ public:
    *\param archive Name of the archive to open.
    *\param mode Access mode, ie. read-only, write-only, etc.
    */
-  ClassStream( const char *dir, const char *archive, const Self::Mode mode )
+  ClassStream( const std::string& dir, const std::string& archive, const Self::Mode mode )
     : TypedStream( dir, archive, mode ) {}
 
   /** Open another archive without constructing a new object.
    */
-  void Open( const char *filename, const Self::Mode mode ) {
+  void Open( const std::string& filename, const Self::Mode mode ) 
+  {
     this->TypedStream::Open( filename, mode );
   }
 
   /** Open another archive in explicit directory.
    */
-  void Open( const char *dir, const char *archive, const Self::Mode mode ) {
+  void Open( const std::string& dir, const std::string& archive, const Self::Mode mode ) 
+  {
     this->TypedStream::Open( dir, archive, mode );
   }
 
@@ -141,20 +143,6 @@ public:
   
   /// Read parametric plane.
   ClassStream& operator >> ( ParametricPlane*& parametricPlane );
-
-  /** Write landmark list.
-   */
-  ClassStream& operator << ( const LandmarkList *landmarkList );
-
-  /** Write landmark list.
-   * This function works on a reference rather than a pointer. It immediately
-   * calls the pointer-based function defined above for the actual writing.
-   */
-  ClassStream& operator << ( const LandmarkList& landmarkList )
-  { return (*this) << &landmarkList; }
-  
-  /// Read landmark list.
-  ClassStream& operator >> ( LandmarkList::SmartPtr& landmarkList );
 };
 
 //@}
