@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3646 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2011-12-22 10:10:55 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -36,11 +36,13 @@
 #include <Base/cmtkSurfaceNormal.h>
 
 #include <System/cmtkConsole.h>
+#include <System/cmtkStrUtility.h>
 
 #include <IO/cmtkFileConstHeader.h>
 
 #ifdef CMTK_USE_DCMTK_JPEG
-#  include <djdecode.h>
+#  include <dcmtk/dcmjpeg/djdecode.h>
+#  include <dcmtk/dcmjpls/djdecode.h>
 #endif
 
 #ifdef HAVE_SYS_TYPES_H
@@ -58,7 +60,7 @@ cmtk
 /** \addtogroup IO */
 //@{
 
-DICOM::DICOM( const char* path )
+DICOM::DICOM( const std::string& path )
 {
 #ifdef CMTK_USE_DCMTK_JPEG
   // register global decompression codecs
@@ -77,7 +79,7 @@ DICOM::DICOM( const char* path )
     }
   
   fileformat->transferInit();
-  OFCondition status = fileformat->loadFile( path );
+  OFCondition status = fileformat->loadFile( path.c_str() );
   fileformat->transferEnd();
 
   if ( !status.good() ) 
@@ -313,7 +315,7 @@ DICOM::GetPixelDataArray( const size_t pixelDataLength )
 
 
 const FixedVector<3,double>
-DICOM::DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imageOrientation, FixedVector<3,int>& dims, TypedArray::SmartPtr& pixelDataArray )
+DICOM::DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imageOrientation, FixedVector<3,int>& dims, TypedArray::SmartPtr& pixelDataArray, FixedVector<3,double>& imageOrigin )
 {
   // without further information, we "guess" the image normal vector
   FixedVector<3,double> sliceNormal = SurfaceNormal( imageOrientation[0], imageOrientation[1] ).Get();
@@ -372,43 +374,8 @@ DICOM::DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imag
 	    }
 	  
 // For the following, see here: http://nipy.sourceforge.net/nibabel/dicom/siemens_csa.html#csa-header
-	  const DcmTagKey csaHeaderInfoTag(0x0029,0x1010);
-	  
-	  const Uint8* csaHeaderInfo = NULL;
-	  unsigned long csaHeaderLength = 0;
-	  this->Dataset().findAndGetUint8Array ( csaHeaderInfoTag, csaHeaderInfo, &csaHeaderLength );
-	  
-	  FileConstHeader fileHeader( csaHeaderInfo, false /*isBigEndian*/ ); // Siemens CSA header is always little endian
-	  const size_t nTags = fileHeader.GetField<Uint32>( 8 );
-	  
-	  size_t tagOffset = 16; // start after header
-	  for ( size_t tag = 0; tag < nTags; ++tag )
-	    {
-	    char tagName[65];
-	    fileHeader.GetFieldString( tagOffset, tagName, 64 );
-//	  StdErr << tag << "\t" << tagName << "\n";
-	    
-	    const size_t nItems = fileHeader.GetField<Uint32>( tagOffset + 76 );
-//	  StdErr << "  nItems: " << nItems << "\n";
-	    
-	    tagOffset += 84;
-	    for ( size_t item = 0; item < nItems; ++item )
-	      {
-	      const size_t itemLen = fileHeader.GetField<Uint32>( tagOffset );
-	      
-//	    StdErr << "    len: " << itemLen << "\n";
-	      
-	      if ( ! strcmp( tagName, "SliceNormalVector" ) && (item < 3) )
-		{
-		char valStr[65];
-		sliceNormal[item] = atof( fileHeader.GetFieldString( tagOffset+16, valStr, 64 ) );
-		
-//	      StdErr << "    " << valStr << "\n";
-		}
-	      
-	      tagOffset += 4*((itemLen+3)/4) /*move up to nearest 4-byte boundary*/ + 16 /*the 4 ints at the beginning of item, including itemLength*/;
-	      }
-	    }
+	  this->ParseSiemensCSA( DcmTagKey(0x0029,0x1010), sliceNormal, imageOrigin ); // image information
+	  this->ParseSiemensCSA( DcmTagKey(0x0029,0x1020), sliceNormal, imageOrigin ); // series information
 	  }
 	}
       }  
@@ -416,6 +383,61 @@ DICOM::DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imag
   return sliceNormal;
 }
 
+void
+DICOM::ParseSiemensCSA( const DcmTagKey& tagKey, FixedVector<3,double>& sliceNormal, FixedVector<3,double>& imageOrigin )
+{
+  const Uint8* csaHeaderInfo = NULL;
+  unsigned long csaHeaderLength = 0;
+  if ( this->Dataset().findAndGetUint8Array ( tagKey, csaHeaderInfo, &csaHeaderLength ).status() == OF_ok )
+    {
+    FileConstHeader fileHeader( csaHeaderInfo, false /*isBigEndian*/ ); // Siemens CSA header is always little endian
+    const size_t nTags = fileHeader.GetField<Uint32>( 8 );
+    
+    size_t tagOffset = 16; // start after header
+    for ( size_t tag = 0; tag < nTags; ++tag )
+      {
+      char tagName[65];
+      fileHeader.GetFieldString( tagOffset, tagName, 64 );
+//      StdErr << tag << "\t" << tagName << "\n";
+      
+      const size_t nItems = fileHeader.GetField<Uint32>( tagOffset + 76 );
+//      StdErr << "  nItems: " << nItems << "\n";
+      
+      tagOffset += 84;
+      for ( size_t item = 0; item < nItems; ++item )
+	{
+	const size_t itemLen = fileHeader.GetField<Uint32>( tagOffset );
+	
+//	StdErr << "    len: " << itemLen << "\n";
+	
+	if ( ! strcmp( tagName, "SliceNormalVector" ) && (item < 3) )
+	  {
+	  char valStr[65];
+	  sliceNormal[item] = atof( fileHeader.GetFieldString( tagOffset+16, valStr, 64 ) );
+	  
+//	  StdErr << "    " << valStr << "\n";
+	  }
+
+	// get true slice0 location from CSA header
+	if ( ! strcmp( tagName, "MrPhoenixProtocol" ) )
+	  {
+	  const char* pSliceZero = StrNStr( reinterpret_cast<const char*>( csaHeaderInfo+tagOffset+16 ), itemLen, "sSliceArray.asSlice[0].sPosition.dSag" );
+	  if ( pSliceZero )
+	    {
+	    for ( int i = 0; i < 3; ++i )
+	      {
+	      while ( *pSliceZero != '=' ) ++pSliceZero;
+	      imageOrigin[i] = atof( ++pSliceZero );
+	      }
+	    }
+	  }
+	
+	tagOffset += 4*((itemLen+3)/4) /*move up to nearest 4-byte boundary*/ + 16 /*the 4 ints at the beginning of item, including itemLength*/;
+	}
+      }
+    }
+}
+
 ScalarImage* 
 DICOM::Read 
 ( const char *path )
