@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3603 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2011-12-01 14:08:18 -0800 (Thu, 01 Dec 2011) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -60,7 +60,7 @@ public:
 
   /** Constructor.
    */
-  DICOM( const char* path );
+  DICOM( const std::string& path );
 
   /// Get image dimensions (number of pixels per axis).
   const FixedVector<3,int> GetDims() const;
@@ -89,7 +89,8 @@ public:
   /// Demosaic (if necessary) the image and return slice normal vector.
   const FixedVector<3,double> DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imageOrientation /*!< Image orientation vectors (read-only, for default normal computation)*/, 
 						    FixedVector<3,int>& dims /*!< Image dimensions - these may be modified if the image is a mosaic */, 
-						    TypedArray::SmartPtr& pixelDataArray /*!< Pixel data array - a new array will be returned if the image is a mosaic.*/ );
+						    TypedArray::SmartPtr& pixelDataArray /*!< Pixel data array - a new array will be returned if the image is a mosaic.*/,
+						    FixedVector<3,double>& imageOrigin /*!< True image origin from CSA header if this is a mosaic file. */);
   
   /// Get const DICOM dataset.
   const DcmDataset& Dataset() const
@@ -125,6 +126,9 @@ private:
 
   /// Pointer to the DICOM document object
   std::auto_ptr<DiDocument> m_Document;
+
+  /// Parse private Siemens CSA data.
+  void ParseSiemensCSA( const DcmTagKey& tagKey, FixedVector<3,double>& sliceNormal, FixedVector<3,double>& imageOrigin );
 };
 
 //@}
