@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3713 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2012-01-10 15:33:21 -0800 (Tue, 10 Jan 2012) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -189,14 +189,14 @@ public:
    *\param filename Name of the archive to open.
    *\param mode Access mode, ie. read-only, write-only, etc.
    */
-  TypedStream( const char* filename, const Self::Mode mode );
+  TypedStream( const std::string& filename, const Self::Mode mode );
 
   /** Open constructor for separate path and archive names.
    *\param dir Directory to open archive in.
    *\param archive Name of the archive to open.
    *\param mode Access mode, ie. read-only, write-only, etc.
    */
-  TypedStream( const char* dir, const char* archive, const Self::Mode mode );
+  TypedStream( const std::string& dir, const std::string& archive, const Self::Mode mode );
 
   /** Destructor.
    * Close() is called to close a possibly open archive.
@@ -205,11 +205,11 @@ public:
 
   /** Open another archive without constructing a new object.
    */
-  void Open( const char* filename, const Self::Mode mode );
+  void Open( const std::string& filename, const Self::Mode mode );
 
   /** Open another archive in explicit directory.
    */
-  void Open( const char* dir, const char* archive, const Self::Mode mode );
+  void Open( const std::string& dir, const std::string& archive, const Self::Mode mode );
 
   /** Close an open archive.
    */
