@@ -1,6 +1,6 @@
 /*
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3604 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2011-12-01 14:16:18 -0800 (Thu, 01 Dec 2011) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -41,7 +41,7 @@ cmtk
 {
 
 const UniformVolume::SmartPtr
-VolumeFromFile::ReadDICOM( const char *path )
+VolumeFromFile::ReadDICOM( const std::string& path )
 {
   DICOM dicom( path );
 
@@ -52,11 +52,11 @@ VolumeFromFile::ReadDICOM( const char *path )
 
   TypedArray::SmartPtr pixelDataArray = dicom.GetPixelDataArray( totalImageSizePixels );
 
-  const UniformVolume::CoordinateVectorType imageOrigin = dicom.GetImageOrigin();
+  UniformVolume::CoordinateVectorType imageOrigin = dicom.GetImageOrigin();
   FixedVector< 2, FixedVector<3,double> > imageOrientation = dicom.GetImageOrientation();
   
   // without further information, we "guess" the image normal vector
-  UniformVolume::CoordinateVectorType sliceNormal = dicom.DemosaicAndGetNormal( imageOrientation, dims, pixelDataArray );
+  UniformVolume::CoordinateVectorType sliceNormal = dicom.DemosaicAndGetNormal( imageOrientation, dims, pixelDataArray, imageOrigin );
 
   // Construct volume and set the DICOM coordinates
   UniformVolume::SmartPtr volume( new UniformVolume( UniformVolume::IndexType( dims ), pixelSize[0], pixelSize[1], pixelSize[2], pixelDataArray ) );
