@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3755 $
+//  $Revision: 4260 $
 //
-//  $LastChangedDate: 2012-01-30 15:29:13 -0800 (Mon, 30 Jan 2012) $
+//  $LastChangedDate: 2012-04-25 16:34:20 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -52,13 +52,13 @@ cmtk
 //@{
 
 Xform::SmartPtr
-XformIO::ReadNrrd( const char* path )
+XformIO::ReadNrrd( const std::string& path )
 {
   DeformationField::SmartPtr dfield( NULL );
   try 
     {
     Nrrd *nrrd = nrrdNew();
-    if ( nrrdLoad( nrrd, path, NULL ) )
+    if ( nrrdLoad( nrrd, path.c_str(), NULL ) )
       throw biffGetDone(NRRD);
 
     if ( nrrd->dim != 4 )
@@ -196,7 +196,7 @@ XformIO::ReadNrrd( const char* path )
 
 void 
 XformIO::WriteNrrd
-( const Xform* xform, const char *path )
+( const Xform* xform, const std::string& path )
 {
   const DeformationField* dfield = dynamic_cast<const DeformationField*>( xform );
   if ( ! dfield )
@@ -268,7 +268,7 @@ XformIO::WriteNrrd
       }
     nrrdAxisInfoSet_nva( nval, nrrdAxisInfoSpaceDirection, spaceDir );
     
-    if ( nrrdSave( path, nval, nios ) )
+    if ( nrrdSave( path.c_str(), nval, nios ) )
       {
       throw( biffGetDone(NRRD) );
       }
