@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3735 $
+//  $Revision: 4093 $
 //
-//  $LastChangedDate: 2012-01-25 12:36:34 -0800 (Wed, 25 Jan 2012) $
+//  $LastChangedDate: 2012-03-27 13:05:25 -0700 (Tue, 27 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -33,7 +33,7 @@
 #include "cmtkElasticRegistration.h"
 
 #include <Base/cmtkLandmarkList.h>
-#include <Base/cmtkMatchedLandmarkList.h>
+#include <Base/cmtkLandmarkPairList.h>
 #include <Base/cmtkUniformVolume.h>
 #include <Base/cmtkSplineWarpXform.h>
 #include <Base/cmtkTypedArrayFunctionHistogramMatching.h>
@@ -104,8 +104,8 @@ ElasticRegistration::InitRegistration ()
     
     if ( sourceLandmarks && targetLandmarks ) 
       {
-      this->m_LandmarkList = MatchedLandmarkList::SmartPtr( new MatchedLandmarkList( sourceLandmarks, targetLandmarks ) );
-      fprintf( stderr, "Matched %d landmarks.\n", (int)this->m_LandmarkList->size() );
+      this->m_LandmarkPairs = LandmarkPairList::SmartPtr( new LandmarkPairList( *(sourceLandmarks), *(targetLandmarks) ) );
+      StdErr << "Matched " << this->m_LandmarkPairs->size() << " landmarks.\n";
       }
     }
   
@@ -242,10 +242,10 @@ ElasticRegistration::MakeFunctional
       newFunctional->SetRigidityConstraintMap( rigidityMap );
       }
     newFunctional->SetGridEnergyWeight( this->m_GridEnergyWeight );
-    if ( this->m_LandmarkList )
+    if ( this->m_LandmarkPairs )
       {
       newFunctional->SetLandmarkErrorWeight( this->m_LandmarkErrorWeight );
-      newFunctional->SetMatchedLandmarkList( this->m_LandmarkList );
+      newFunctional->SetLandmarkPairs( this->m_LandmarkPairs );
       }
     
     return newFunctional;
