@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3939 $
+//  $Revision: 4093 $
 //
-//  $LastChangedDate: 2012-02-28 15:53:25 -0800 (Tue, 28 Feb 2012) $
+//  $LastChangedDate: 2012-03-27 13:05:25 -0700 (Tue, 27 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -38,7 +38,7 @@
 #include <Registration/cmtkVoxelRegistration.h>
 
 #include <Base/cmtkSplineWarpXform.h>
-#include <Base/cmtkMatchedLandmarkList.h>
+#include <Base/cmtkLandmarkPairList.h>
 
 namespace
 cmtk
@@ -51,7 +51,7 @@ cmtk
  * By implementing member functions to retrieve parameters and report results
  * in derived classes, registration can be integrated into various 
  * environments.
- *\version $Revision: 3939 $ $Date: 2012-02-28 15:53:25 -0800 (Tue, 28 Feb 2012) $
+ *\version $Revision: 4093 $ $Date: 2012-03-27 13:05:25 -0700 (Tue, 27 Mar 2012) $
  */
 class ElasticRegistration : 
   /// Inherit basic voxel registration functions.
@@ -143,7 +143,7 @@ protected:
   cmtkGetSetMacro(float,LandmarkErrorWeight);
 
   /// Matched landmarks list.
-  cmtkGetSetMacro(MatchedLandmarkList::SmartPtr,LandmarkList);
+  cmtkGetSetMacro(LandmarkPairList::SmartPtr,LandmarkPairs);
 
   /// Flag to turn on deformation unfolding before each level.
   bool m_RelaxToUnfold;
