@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3499 $
+//  $Revision: 3647 $
 //
-//  $LastChangedDate: 2011-10-21 12:48:54 -0700 (Fri, 21 Oct 2011) $
+//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
 //
-//  $LastChangedBy: torsten_at_home $
+//  $LastChangedBy: torstenrohlfing $
 //
 */
 
@@ -248,7 +248,7 @@ ElasticRegistrationCommandLine
     
     DebugOutput( 1 ) << "Reading input studylist " << InputStudylist << "\n";
   
-    ClassStream classStream( MountPoints::Translate(InputStudylist),"registration", ClassStream::READ );
+    ClassStream classStream( MountPoints::Translate(InputStudylist),"registration", ClassStream::MODE_READ );
     if ( ! classStream.IsValid() ) 
       {
       StdErr << "ERROR: Could not open studylist archive " << InputStudylist << ".\n";
@@ -458,7 +458,7 @@ ElasticRegistrationCommandLine::DoneResolution
 void
 ElasticRegistrationCommandLine::OutputWarp ( const char* path ) const
 {
-  ClassStream classStream( path, "studylist", ClassStream::WRITE );
+  ClassStream classStream( path, "studylist", ClassStream::MODE_WRITE );
   if ( ! classStream.IsValid() ) return;
 
   classStream.Begin( "studylist" );
@@ -475,7 +475,7 @@ ElasticRegistrationCommandLine::OutputWarp ( const char* path ) const
 
   classStream.Close();
 
-  classStream.Open( path, "settings", ClassStream::WRITE );
+  classStream.Open( path, "settings", ClassStream::MODE_WRITE );
   classStream.WriteInt( "algorithm", this->m_Algorithm );
   classStream.WriteBool( "use_maxnorm", UseMaxNorm );
   classStream.WriteDouble( "exploration", this->m_Exploration );
@@ -509,7 +509,7 @@ ElasticRegistrationCommandLine::OutputWarp ( const char* path ) const
 
   classStream.Close();
       
-  classStream.Open( path, "statistics", ClassStream::WRITE );
+  classStream.Open( path, "statistics", ClassStream::MODE_WRITE );
   classStream.WriteDouble( "time_level", this->GetLevelElapsedTime() );
   classStream.WriteDouble( "time_total", this->GetTotalElapsedTime() );
   classStream.WriteDouble( "walltime_level", this->GetLevelElapsedWalltime() );
@@ -532,7 +532,7 @@ ElasticRegistrationCommandLine::OutputWarp ( const char* path ) const
   const WarpXform::SmartPtr warp = WarpXform::SmartPtr::DynamicCastFrom( this->m_Xform );
   if ( warp ) 
     {
-    classStream.Open( path, "registration", ClassStream::WRITE );
+    classStream.Open( path, "registration", ClassStream::MODE_WRITE );
     if ( classStream.IsValid() ) 
       {
       classStream.Begin( "registration" );
