@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3608 $
+//  $Revision: 4111 $
 //
-//  $LastChangedDate: 2011-12-02 10:47:18 -0800 (Fri, 02 Dec 2011) $
+//  $LastChangedDate: 2012-03-30 14:30:21 -0700 (Fri, 30 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -41,7 +41,7 @@
 
 #include <Base/cmtkInterpolator.h>
 #include <Base/cmtkTypedArrayFunctionHistogramMatching.h>
-#include <Base/cmtkUniformVolumeFilter.h>
+#include <Base/cmtkUniformVolumeGaussianFilter.h>
 #include <Registration/cmtkReformatVolume.h>
 
 namespace
@@ -243,7 +243,7 @@ GroupwiseRegistrationFunctionalBase
   TypedArray::SmartPtr data;
   if ( this->m_GaussianSmoothImagesSigma > 0 )
     {
-    data = UniformVolumeFilter( image ).GetDataGaussFiltered( Units::GaussianSigma( this->m_GaussianSmoothImagesSigma * this->m_TemplateGrid->GetMinDelta() ) );
+    data = UniformVolumeGaussianFilter( image ).GetFiltered3D( Units::GaussianSigma( this->m_GaussianSmoothImagesSigma * this->m_TemplateGrid->GetMinDelta() ) );
     
     if ( this->m_FreeAndRereadImages )
       {
