@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3735 $
+//  $Revision: 4093 $
 //
-//  $LastChangedDate: 2012-01-25 12:36:34 -0800 (Wed, 25 Jan 2012) $
+//  $LastChangedDate: 2012-03-27 13:05:25 -0700 (Tue, 27 Mar 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -33,7 +33,7 @@
 #include "cmtkImagePairNonrigidRegistration.h"
 
 #include <Base/cmtkLandmarkList.h>
-#include <Base/cmtkMatchedLandmarkList.h>
+#include <Base/cmtkLandmarkPairList.h>
 #include <Base/cmtkUniformVolume.h>
 #include <Base/cmtkSplineWarpXform.h>
 #include <Base/cmtkTypedArrayFunctionHistogramMatching.h>
@@ -92,7 +92,6 @@ ImagePairNonrigidRegistration::InitRegistration ()
   this->m_ReferenceVolume = this->m_Volume_1;
   this->m_FloatingVolume = this->m_Volume_2;
 
-  MatchedLandmarkList::SmartPtr mll( NULL );
   if ( this->m_LandmarkErrorWeight != 0 ) 
     {
     LandmarkList::SmartPtr sourceLandmarks = this->m_ReferenceVolume->m_LandmarkList;
@@ -100,8 +99,8 @@ ImagePairNonrigidRegistration::InitRegistration ()
     
     if ( sourceLandmarks && targetLandmarks ) 
       {
-      this->m_MatchedLandmarks = MatchedLandmarkList::SmartPtr( new MatchedLandmarkList( sourceLandmarks, targetLandmarks ) );
-      fprintf( stderr, "Matched %d landmarks.\n", (int)this->m_MatchedLandmarks->size() );
+      this->m_LandmarkPairs = LandmarkPairList::SmartPtr( new LandmarkPairList( *sourceLandmarks, *targetLandmarks ) );
+      cmtk::StdErr << "Matched " <<this->m_LandmarkPairs->size() << "landmarks.\n";
       }
     }
   
@@ -251,10 +250,10 @@ ImagePairNonrigidRegistration::MakeFunctional
     newFunctional->SetJacobianConstraintWeight( this->m_JacobianConstraintWeight );
     newFunctional->SetForceOutside( this->m_ForceOutsideFlag, this->m_ForceOutsideValue );
     newFunctional->SetGridEnergyWeight( this->m_GridEnergyWeight );
-    if ( this->m_MatchedLandmarks )
+    if ( this->m_LandmarkPairs )
       {
       newFunctional->SetLandmarkErrorWeight( this->m_LandmarkErrorWeight );
-      newFunctional->SetMatchedLandmarkList( this->m_MatchedLandmarks );
+      newFunctional->SetLandmarkPairs( this->m_LandmarkPairs );
       }
     
     return newFunctional;
