@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3001 $
+//  $Revision: 3647 $
 //
-//  $LastChangedDate: 2011-03-16 12:14:22 -0700 (Wed, 16 Mar 2011) $
+//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -233,7 +233,7 @@ cmtk::ImageSymmetryPlaneCommandLineBase
   
   if ( this->m_SymmetryOutFileName )
     {
-    ClassStream stream( this->m_SymmetryOutFileName, ClassStream::WRITE );
+    ClassStream stream( this->m_SymmetryOutFileName, ClassStream::MODE_WRITE );
     stream << this->m_SymmetryPlane;
     stream.Close();
     }
@@ -280,7 +280,7 @@ cmtk::ImageSymmetryPlaneCommandLineBase
     
     if ( this->m_SymmetryParametersFile ) 
       {
-      ClassStream inStream( this->m_SymmetryParametersFile, ClassStream::READ );
+      ClassStream inStream( this->m_SymmetryParametersFile, ClassStream::MODE_READ );
       if ( inStream.IsValid() ) 
 	{
 	ParametricPlane *plane = NULL;
