@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3821 $
+//  $Revision: 4263 $
 //
-//  $LastChangedDate: 2012-02-07 12:43:33 -0800 (Tue, 07 Feb 2012) $
+//  $LastChangedDate: 2012-04-25 17:10:54 -0700 (Wed, 25 Apr 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -32,10 +32,6 @@
 
 #include "cmtkVoxelMatchingElasticFunctional.h"
 
-#ifdef CMTK_USE_SMP
-#  include <Registration/cmtkParallelElasticFunctional.h>
-#endif
-
 #include <System/cmtkDebugOutput.h>
 
 #include <Registration/cmtkVoxelMatchingMutInf.h>
@@ -49,6 +45,10 @@
 
 #include <vector>
 
+#ifdef _OPENMP
+#  include <omp.h>
+#endif
+
 namespace
 cmtk
 {
@@ -122,10 +122,10 @@ VoxelMatchingElasticFunctional_WarpTemplate<W>::WeightedDerivative
     }
   else
     {
-    if ( this->m_MatchedLandmarkList.GetPtr() ) 
+    if ( this->m_LandmarkPairs ) 
       {
       double lowerMSD, upperMSD;
-      warp.GetDerivativeLandmarksMSD( lowerMSD, upperMSD, this->m_MatchedLandmarkList, param, step );
+      warp.GetDerivativeLandmarksMSD( lowerMSD, upperMSD, *(this->m_LandmarkPairs), param, step );
       lower -= this->m_LandmarkErrorWeight * lowerMSD;
       upper -= this->m_LandmarkErrorWeight * upperMSD;
       }
@@ -173,30 +173,11 @@ template<class VM>
 void
 VoxelMatchingElasticFunctional_Template<VM>::UpdateWarpFixedParameters() 
 {
-  if ( !this->ConsistencyHistogram ) 
-    {
-    this->ConsistencyHistogram = JointHistogram<unsigned int>::SmartPtr( new JointHistogram<unsigned int>() );
-    const unsigned int numSamplesX = this->Metric->DataX.NumberOfSamples;
-    const Types::DataItemRange rangeX = this->Metric->DataX.GetValueRange();
-    const unsigned int numBinsX = this->ConsistencyHistogram->CalcNumBins( numSamplesX, rangeX );
-    
-    const unsigned int numSamplesY = this->Metric->DataY.NumberOfSamples;
-    const Types::DataItemRange rangeY = this->Metric->DataY.GetValueRange();
-    unsigned int numBinsY = this->ConsistencyHistogram->CalcNumBins( numSamplesY, rangeY );
-    
-    this->ConsistencyHistogram->Resize( numBinsX, numBinsY );
-    this->ConsistencyHistogram->SetRangeX( rangeX );
-    this->ConsistencyHistogram->SetRangeY( rangeY );
-    }
-  
   int numCtrlPoints = this->Dim / 3;
   
   std::vector<double> mapRef( numCtrlPoints );
   std::vector<double> mapMod( numCtrlPoints );
 
-  Vector3D fromVOI, toVOI;
-  int pX, pY, pZ;
-
   int inactive = 0;
 
   const typename VM::Exchange unsetY = this->Metric->DataY.padding();
@@ -207,7 +188,8 @@ VoxelMatchingElasticFunctional_Template<VM>::UpdateWarpFixedParameters()
       this->Warp->SetParametersActive( this->m_ActiveCoordinates );
     else
       this->Warp->SetParametersActive();
-    
+
+#pragma omp parallel for reduction(+:inactive)
     for ( int ctrl = 0; ctrl < numCtrlPoints; ++ctrl ) 
       {
       /// We cannot use the precomputed table of VOIs here because in "fast" mode, these VOIs are smaller than we want them here.
@@ -216,11 +198,11 @@ VoxelMatchingElasticFunctional_Template<VM>::UpdateWarpFixedParameters()
       int r = voi.From()[0] + this->DimsX * ( voi.From()[1] + this->DimsY * voi.From()[2] );
       
       bool active = false;
-      for ( pZ = voi.From()[2]; (pZ < voi.To()[2]) && !active; ++pZ ) 
+      for ( int pZ = voi.From()[2]; (pZ < voi.To()[2]) && !active; ++pZ ) 
 	{
-	for ( pY = voi.From()[1]; (pY < voi.To()[1]) && !active; ++pY ) 
+	for ( int pY = voi.From()[1]; (pY < voi.To()[1]) && !active; ++pY ) 
 	  {
-	  for ( pX = voi.From()[0]; (pX < voi.To()[0]); ++pX, ++r ) 
+	  for ( int pX = voi.From()[0]; (pX < voi.To()[0]); ++pX, ++r ) 
 	    {
 	    if ( ( this->Metric->GetSampleX( r ) != 0 ) || ( ( this->WarpedVolume[r] != unsetY ) && ( this->WarpedVolume[r] != 0 ) ) ) 
 	      {
@@ -247,74 +229,119 @@ VoxelMatchingElasticFunctional_Template<VM>::UpdateWarpFixedParameters()
     } 
   else
     {
-    for ( int ctrl = 0; ctrl < numCtrlPoints; ++ctrl ) 
+#ifdef _OPENMP
+    if ( this->m_ThreadConsistencyHistograms.size() < static_cast<size_t>( omp_get_max_threads() ) )
       {
-      this->ConsistencyHistogram->Reset();
+      this->m_ThreadConsistencyHistograms.resize( omp_get_max_threads() );
       
-      // We cannot use the precomputed table of VOIs here because in "fast" mode, these VOIs are smaller than we want them here.
-      const DataGrid::RegionType voi = this->GetReferenceGridRange( this->Warp->GetVolumeOfInfluence( 3 * ctrl, this->m_ReferenceDomain, false /* disable fast mode */ ) );
+      const unsigned int numSamplesX = this->Metric->DataX.NumberOfSamples;
+      const Types::DataItemRange rangeX = this->Metric->DataX.GetValueRange();
+      const unsigned int numBinsX = JointHistogramBase::CalcNumBins( numSamplesX, rangeX );
+
+      const unsigned int numSamplesY = this->Metric->DataY.NumberOfSamples;
+      const Types::DataItemRange rangeY = this->Metric->DataY.GetValueRange();
+      const unsigned int numBinsY = JointHistogramBase::CalcNumBins( numSamplesY, rangeY );
       
-      int r = voi.From()[0] + this->DimsX * ( voi.From()[1] + this->DimsY * voi.From()[2] );
-      
-      const int endOfLine = ( voi.From()[0] + ( this->DimsX-voi.To()[0]) );
-      const int endOfPlane = this->DimsX * ( voi.From()[1] + (this->DimsY-voi.To()[1]) );
-      
-      for ( pZ = voi.From()[2]; pZ<voi.To()[2]; ++pZ ) 
+      for ( size_t thread = 0; thread < static_cast<size_t>( omp_get_max_threads() ); ++thread )
 	{
-	for ( pY = voi.From()[1]; pY<voi.To()[1]; ++pY ) 
+	if ( ! this->m_ThreadConsistencyHistograms[thread] )
 	  {
-	  for ( pX = voi.From()[0]; pX<voi.To()[0]; ++pX, ++r ) 
+	  this->m_ThreadConsistencyHistograms[thread] = JointHistogram<unsigned int>::SmartPtr( new JointHistogram<unsigned int>() );
+	  
+	  this->m_ThreadConsistencyHistograms[thread]->Resize( numBinsX, numBinsY );
+	  this->m_ThreadConsistencyHistograms[thread]->SetRangeX( rangeX );
+	  this->m_ThreadConsistencyHistograms[thread]->SetRangeY( rangeY );
+	  }
+	}
+      }
+#else
+      const unsigned int numSamplesX = this->Metric->DataX.NumberOfSamples;
+      const Types::DataItemRange rangeX = this->Metric->DataX.GetValueRange();
+      const unsigned int numBinsX = JointHistogramBase::CalcNumBins( numSamplesX, rangeX );
+
+      const unsigned int numSamplesY = this->Metric->DataY.NumberOfSamples;
+      const Types::DataItemRange rangeY = this->Metric->DataY.GetValueRange();
+      const unsigned int numBinsY = JointHistogramBase::CalcNumBins( numSamplesY, rangeY );
+      
+      this->m_ConsistencyHistogram = JointHistogram<unsigned int>::SmartPtr( new JointHistogram<unsigned int>() );
+      
+      this->m_ConsistencyHistogram->Resize( numBinsX, numBinsY );
+      this->m_ConsistencyHistogram->SetRangeX( rangeX );
+      this->m_ConsistencyHistogram->SetRangeY( rangeY );
+#endif
+      
+#pragma omp parallel for
+      for ( int ctrl = 0; ctrl < numCtrlPoints; ++ctrl ) 
+	{
+#ifdef _OPENMP
+	JointHistogram<unsigned int>& threadHistogram = *(this->m_ThreadConsistencyHistograms[ omp_get_thread_num() ]);
+#else
+	JointHistogram<unsigned int>& threadHistogram = *(this->m_ConsistencyHistogram);
+#endif
+	threadHistogram.Reset();
+	
+	// We cannot use the precomputed table of VOIs here because in "fast" mode, these VOIs are smaller than we want them here.
+	const DataGrid::RegionType voi = this->GetReferenceGridRange( this->Warp->GetVolumeOfInfluence( 3 * ctrl, this->m_ReferenceDomain, false /* disable fast mode */ ) );
+	
+	int r = voi.From()[0] + this->DimsX * ( voi.From()[1] + this->DimsY * voi.From()[2] );
+	
+	const int endOfLine = ( voi.From()[0] + ( this->DimsX-voi.To()[0]) );
+	const int endOfPlane = this->DimsX * ( voi.From()[1] + (this->DimsY-voi.To()[1]) );
+	
+	for ( int pZ = voi.From()[2]; pZ<voi.To()[2]; ++pZ ) 
+	  {
+	  for ( int pY = voi.From()[1]; pY<voi.To()[1]; ++pY ) 
 	    {
-	    // Continue metric computation.
-	    if ( WarpedVolume[r] != unsetY ) 
+	    for ( int pX = voi.From()[0]; pX<voi.To()[0]; ++pX, ++r ) 
 	      {
-	      this->ConsistencyHistogram->Increment
-		( this->ConsistencyHistogram->ValueToBinX( this->Metric->GetSampleX( r ) ), 
-		  this->ConsistencyHistogram->ValueToBinY( this->WarpedVolume[r] ) );
+	      // Continue metric computation.
+	      if ( this->WarpedVolume[r] != unsetY ) 
+		{
+		threadHistogram.Increment( threadHistogram.ValueToBinX( this->Metric->GetSampleX( r ) ), threadHistogram.ValueToBinY( this->WarpedVolume[r] ) );
+		}
 	      }
+	    r += endOfLine;
 	    }
-	  r += endOfLine;
+	  r += endOfPlane;
 	  }
-	r += endOfPlane;
+	threadHistogram.GetMarginalEntropies( mapRef[ctrl], mapMod[ctrl] );
 	}
-      this->ConsistencyHistogram->GetMarginalEntropies( mapRef[ctrl], mapMod[ctrl] );
-      }
-    
-    double refMin = HUGE_VAL, refMax = -HUGE_VAL;
-    double modMin = HUGE_VAL, modMax = -HUGE_VAL;
-    for ( int ctrl=0; ctrl<numCtrlPoints; ++ctrl ) 
-      {
-      if ( mapRef[ctrl] < refMin ) refMin = mapRef[ctrl];
-      if ( mapRef[ctrl] > refMax ) refMax = mapRef[ctrl];
-      if ( mapMod[ctrl] < modMin ) modMin = mapMod[ctrl];
-      if ( mapMod[ctrl] > modMax ) modMax = mapMod[ctrl];
-      }
-    
-    const double refThresh = refMin + this->m_AdaptiveFixThreshFactor * (refMax - refMin);
-    const double modThresh = modMin + this->m_AdaptiveFixThreshFactor * (modMax - modMin);
       
-    if ( this->m_ActiveCoordinates )
-      this->Warp->SetParametersActive( this->m_ActiveCoordinates );
-    else
-      this->Warp->SetParametersActive();
-          
-    for ( int ctrl=0; ctrl<numCtrlPoints; ++ctrl ) 
-      {
-      if (  ( mapRef[ctrl] < refThresh ) && ( mapMod[ctrl] < modThresh ) ) 
+      double refMin = HUGE_VAL, refMax = -HUGE_VAL;
+      double modMin = HUGE_VAL, modMax = -HUGE_VAL;
+      for ( int ctrl=0; ctrl<numCtrlPoints; ++ctrl ) 
 	{
-	int dim = 3 * ctrl;
-	for ( int idx=0; idx<3; ++idx, ++dim ) 
+	if ( mapRef[ctrl] < refMin ) refMin = mapRef[ctrl];
+	if ( mapRef[ctrl] > refMax ) refMax = mapRef[ctrl];
+	if ( mapMod[ctrl] < modMin ) modMin = mapMod[ctrl];
+	if ( mapMod[ctrl] > modMax ) modMax = mapMod[ctrl];
+	}
+      
+      const double refThresh = refMin + this->m_AdaptiveFixThreshFactor * (refMax - refMin);
+      const double modThresh = modMin + this->m_AdaptiveFixThreshFactor * (modMax - modMin);
+      
+      if ( this->m_ActiveCoordinates )
+	this->Warp->SetParametersActive( this->m_ActiveCoordinates );
+      else
+	this->Warp->SetParametersActive();
+      
+      for ( int ctrl=0; ctrl<numCtrlPoints; ++ctrl ) 
+	{
+	if (  ( mapRef[ctrl] < refThresh ) && ( mapMod[ctrl] < modThresh ) ) 
 	  {
-	  this->Warp->SetParameterInactive( dim );
+	  int dim = 3 * ctrl;
+	  for ( int idx=0; idx<3; ++idx, ++dim ) 
+	    {
+	    this->Warp->SetParameterInactive( dim );
+	    }
+	  inactive += 3;
 	  }
-	inactive += 3;
 	}
-      }
     }
   
   for ( size_t idx = 0; idx < this->Dim; ++idx ) 
     {
-
+    
     if ( this->Warp->GetParameterActive( idx ) )
       {
       this->StepScaleVector[idx] = this->GetParamStep( idx );
@@ -336,50 +363,6 @@ CreateElasticFunctional
   UniformVolume::SmartPtr& refVolume, 
   UniformVolume::SmartPtr& fltVolume )
 {
-#ifdef CMTK_USE_SMP
-  switch ( fltVolume->GetData()->GetDataClass() ) 
-    {
-    case DATACLASS_UNKNOWN :
-    case DATACLASS_GREY :
-      switch ( metric ) 
-	{
-	case 0:
-	  return new ParallelElasticFunctional< VoxelMatchingNormMutInf_Trilinear>( refVolume, fltVolume );
-	case 1:
-	  return new ParallelElasticFunctional<VoxelMatchingMutInf_Trilinear>( refVolume, fltVolume );
-	case 2:
-	  return new ParallelElasticFunctional<VoxelMatchingCorrRatio_Trilinear>( refVolume, fltVolume );
-	case 3:
-	  return NULL; // masked nmi retired
-	case 4:
-	  return new ParallelElasticFunctional<VoxelMatchingMeanSquaredDifference>( refVolume, fltVolume );
-	case 5:
-	  return new ParallelElasticFunctional<VoxelMatchingCrossCorrelation>( refVolume, fltVolume );
-	default:
-	  return NULL;
-	}
-    case DATACLASS_LABEL:
-      switch ( metric ) 
-	{
-	case 0:
-	  return new ParallelElasticFunctional< VoxelMatchingNormMutInf_NearestNeighbor >( refVolume, fltVolume );
-	case 1:
-	  return new ParallelElasticFunctional<VoxelMatchingMutInf_NearestNeighbor>( refVolume, fltVolume );
-	case 2:
-	  return new ParallelElasticFunctional<VoxelMatchingCorrRatio_NearestNeighbor>( refVolume, fltVolume );
-	case 3:
-	  return NULL; // masked nmi retired
-	case 4:
-	  return new ParallelElasticFunctional<VoxelMatchingMeanSquaredDifference>( refVolume, fltVolume );
-	case 5:
-	  return new ParallelElasticFunctional<VoxelMatchingCrossCorrelation>( refVolume, fltVolume );
-	default:
-	  return NULL;
-	}
-    }
-
-#else // single processing
-
   switch ( fltVolume->GetData()->GetDataClass() ) 
     {
     case DATACLASS_UNKNOWN :
@@ -420,7 +403,6 @@ CreateElasticFunctional
 	return NULL;
 	}
     }
-#endif
   
   return NULL;
 }
